/**
 * Created on Jun 20, 2005
 * This will index the new J drive or what ever directories choosen.
 * 
 * Copyright 2005 Tom Louton, Berlin, Germany, http://www.theloutons.com/photoalbumtom.htm 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.theloutons.search;
import java.io.File;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.io.IOException;
import java.util.Date;
import java.util.Properties;

import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexWriter;

import com.theloutons.search.utils.*;

/**
 * @author Tom
 * 
 * This recursively searches the J drive for files it can read
 * viz txt, pdf, doc(hopefully Winword),HTML (maybe XML) This
 * is started from a dos window
 *  sample start config. C:/temp/pptIndexTest  c:/ppTest/excel  c:/temp/ppt.log
 */
public class JDriveIndexer {

	/**
	 * @param args[0] the drive for the index. 
	 * @param args[1] the sorce of the data. 
	 * BE CAREFUL it removes all files from args[0]!
	 * an exception is thrown if the args are not there, 
	 * both of them.
	 * a third args[2] is optionally possible to log file which could 
	 * not be indexed.
	 * c:/lucene/tmpindex  C:/OpenNLP/sandbox
	 * c:/lucene/tmpindex  C:/OpenNLP/sandbox c:/lucene/index.log
	 */
    private static FileWriter logWriter;
    private static PrintWriter log;
	public static void main(String[] args) throws Exception {
		if (args.length < 2) {
			throw new Exception("Usage: java " + JDriveIndexer.class.getName()
					+ "<index dir> <data dir>" + " Your parameters "
					+ args.length);
		}
		String logFileName="jdriveIndexer.log";
		if (args.length ==3) {logFileName=args[2];}
		//C:/SAS-Data/utf-8 good test dir.
		File logFile = new File(logFileName); 
		logWriter = new FileWriter(logFile);
		log = new PrintWriter(logWriter);
		log.println("Indexing "+args[1]+ "with "+args[0]+ " as the target directoy for the index.\n");
		File indexDir = new File(args[0]);
		File dataDir = new File(args[1]);
		long start = new Date().getTime();
		log.println("Started at " +start + " --  measured in java time.\n"); 
		log.println("The following could not be indexed.\n");
		int numIndexed = index(indexDir, dataDir);
		long end = new Date().getTime();
		System.out.println("Indexing " + numIndexed + " files took "
				+ (end - start) + " milliseconds.");
		log.println("Ended at " +end + "   measured in java time.\n"); 
		log.close();
	}

	// open an index and start file directory traversal
	public static int index(File indexDir, File dataDir) throws IOException {
		if (!dataDir.exists() || !dataDir.isDirectory()) {
			throw new IOException(dataDir
					+ " does not exist or is not a directory");
		}
		StopWordMgmt def = new StopWordMgmt();
		String[] stopWords=def.getDefault();

		IndexWriter writer = new IndexWriter(indexDir, new StandardAnalyzer(stopWords),
				true);
		writer.setUseCompoundFile(false);

		indexDirectory(writer, dataDir);

		int numIndexed = writer.docCount();
		writer.optimize();
		writer.close();
		return numIndexed;
	}

	// recursive method that calls itself when it finds a directory
	private static void indexDirectory(IndexWriter writer, File dir)
			throws IOException {
		File[] files = dir.listFiles();
		if (files != null) {
			for (int lauf = 0; lauf < files.length; lauf++) {
				File f = files[lauf];
				if (f.isDirectory()) {
					indexDirectory(writer, f);
				}

				else  {
					indexFile(writer, f);
				}
			} // if
		}
	}

	// the method to actually index the file
	private static void indexFile(IndexWriter writer, File f)
			throws IOException {
		if (f.isHidden() || !f.exists() || !f.canRead()) {
			return;
		}
		Properties def = new Properties();
		// need the dummy thing 
		System.out.println("Indexing " + f.getCanonicalPath());

		Document doc = DocReadOrgainzer.getDocument(f, def,log);
		if (doc != null) {
			System.out.println("Adding " + f.getCanonicalPath());
			writer.addDocument(doc);
		}
		else {
		    log.println(f.getCanonicalPath()+"  could not be indexed.\n");
		}
	}
}
