/*
 * Created on Aug 30, 2005
 * Copyright 2005 Tom Louton; Berlin, Germany, http://www.theloutons.com/photoalbumtom.htm 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.theloutons.search.specialreaders;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;

import com.theloutons.search.utils.SynopsisUtil;


/**
 * @author Tom Louton
 *         <p>
 *         The will read csv files (comma separated variables) and its main task
 *         will be to filter to get really only words and skip the cells with
 *         numbers in them.
 *         </p>
 */
public class CSVReader implements DocumentAnalyze {
    private PrintWriter log;

    private Document doc = null;

    /**
     *  
     */
    public CSVReader() {
        super();
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.schering.ci.search.interfaces.DocumentAnalyze#getDocument()
     */
    public Document getDocument() {
        return this.doc;
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.schering.ci.search.interfaces.DocumentAnalyze#setFile(java.io.File)
     */
    public void setFile(File f, PrintWriter log) {
        if (doc == null) {
            this.doc = new org.apache.lucene.document.Document();
        }
        this.log = log;
        try {
            FileInputStream fIS = new FileInputStream(f);
            InputStreamReader isRead = new InputStreamReader(fIS, "UTF-8");
            BufferedReader bR = new BufferedReader(isRead);
            String docTxt = "";
            String line;
            while ((line = bR.readLine()) != null) {
                docTxt = docTxt + filterCommas(line, ',');
            }

            Reader strReader = new StringReader(docTxt);
            doc.add(Field.Text("contents", strReader));
            // how reasonable from EXCEL
            String synop = SynopsisUtil.getSynopsis(docTxt);
            if (synop != null) {
                doc.add(Field.Keyword("synopsis", synop));
            }

        } catch (UnsupportedEncodingException ueE) {
            System.out.println("Unsupported Encoding" + f.getName());
            log.println("Unsupported encoding" + f.getName());
            this.doc = null;
            return;
        } catch (IOException ioE) {
            System.out.println("IO Exception " + f.getName());
            log.println("IO Exception" + f.getName());
            this.doc = null;
            return;
        }

    }

    /**
     * The first step is to generate an array for the cells, then go through the
     * array and look for numbers, keep all else.
     * 
     * @param raw
     *            the string separated by comma, semicolon etc.
     * @return the filtered string.
     */
    private String filterCommas(String raw, char marker) {
        String rt = "";
        boolean didit = false;
        int start = 0;
        int lstIndx = raw.lastIndexOf(marker);
        int idx = raw.indexOf(marker, start);
        if (idx > 0) {
            while (idx < lstIndx) {
                try {
                    Integer i = Integer.valueOf(raw.substring(start, idx));
                } catch (NumberFormatException nfE) {
                    rt = rt + " " + raw.substring(start, idx);
                }
                start = idx + 1;
                idx = raw.indexOf(marker, start);
            }
            // if does not end in , then get the last bit.
            if (raw.length() > idx) {
                // this stops just before the last interval. so get it
                // then position from last comma to end
                try {
                    Integer i = Integer.valueOf(raw.substring(start, idx));
                } catch (NumberFormatException nfE) {
                    rt = rt + " " + raw.substring(start, idx);
                }

                try {
                    Integer i = Integer.valueOf(raw.substring(idx + 1));
                } catch (NumberFormatException nfE) {
                    rt = rt + " " + raw.substring(idx + 1);
                }
            }
        }
        return rt;
    }
    public String toString(){
    	String name = this.getClass().getName();
    	return name + " Reads CSV (comma separated dcouments) the standard Excel export format. Note that in Europe because comma is the separater for decemial place, a semicolon is usually used. I have not properlly addressed this.";
    	
    }
}
