/*
 * Created on Sep 1, 2005
 * Copyright 2005 Tom Louton; Berlin, Germany, http://www.theloutons.com/photoalbumtom.htm 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.theloutons.search.specialreaders;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.PrintWriter;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.textmining.text.extraction.WordExtractor;

import com.theloutons.search.utils.SynopsisUtil;

/**
 * @author Tom Louton
 *
 */
public class DOCReader implements DocumentAnalyze {
    private PrintWriter log;
    private Document doc=null;

    /**
     * 
     */
    public DOCReader() {
        super();
    }

    /* (non-Javadoc)
     * @see de.schering.ci.search.specialreaders.DocumentAnalyze#getDocument()
     */
    public Document getDocument() {
        return this.doc;
    }

    /* (non-Javadoc)
     * @see de.schering.ci.search.specialreaders.DocumentAnalyze#setFile(java.io.File, java.io.PrintWriter)
     */
    public void setFile(File f, PrintWriter log) {
      this.log=log;
      doc= new Document();
      String docStr = getWWDocReader(f);
      
      SynopsisUtil.addString(docStr, doc);
      if (docStr != null) {
          doc.add(Field.Keyword("docType", "doc"));
      }
    }
    /**
     * The reads a winword doc. 
     * @param f the file containing the 
     * @return
     */
    private String getWWDocReader(File f) {
        int sizeofWWDoc = (int) f.length();
        byte[] data = new byte[sizeofWWDoc];
        try {
            DataInputStream in = new DataInputStream(new FileInputStream(f));
            //in.readFully(data);
            WordExtractor wE = new WordExtractor();
            String rtValue = wE.extractText(in);
            wE = null;
            in = null;
            return rtValue;

        }
        /**
         * if something goes wrong, just return null and forget it.
         */
        catch (FileNotFoundException fnfE) {
            log.println(fnfE.getMessage());
            return null;
        } catch (Exception eE) {
            log.println(eE.getMessage());
            return null;
        }
    }
    public String toString(){
    	String name = this.getClass().getName();
    	return name + " Reads WinWord dcouments (doc).";
    	
    }

}
