/*
 * Created on Sep 1, 2005
 * Copyright 2005 Tom Louton; Berlin, Germany, http://www.theloutons.com/photoalbumtom.htm 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. *
 */
package com.theloutons.search.specialreaders;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;
import org.w3c.tidy.Tidy;

import com.theloutons.search.utils.SynopsisUtil;


/**
 * @author Tom Louton
 *
 */
public class HTMLReader implements DocumentAnalyze {
private Document doc=null;
private PrintWriter log;
    /**
     * 
     */
    public HTMLReader() {
        super();
    }

    /* (non-Javadoc)
     * @see de.schering.ci.search.specialreaders.DocumentAnalyze#getDocument()
     */
    public Document getDocument() {
        return this.doc;
    }

    /* (non-Javadoc)
     * @see de.schering.ci.search.specialreaders.DocumentAnalyze#setFile(java.io.File, java.io.PrintWriter)
     */
    public void setFile(File f, PrintWriter log) {
        this.log=log;
        doc= new Document();
        doc.add(Field.Keyword("docType", "html"));
        try {
            InputStream inputStr = new FileInputStream(f);
            InputStreamReader inputRd = new InputStreamReader(inputStr,
                    "UTF-8");

            org.w3c.tidy.Tidy tidy = new Tidy();
            tidy.setMakeClean(true); // Ohne St�rungen
            tidy.setXmlTags(true); // Eingabe als XML behandeln

            org.w3c.dom.Document root = tidy.parseDOM(inputRd, null);
            Element rawDoc = root.getDocumentElement();
            String title = getTitle(rawDoc);
            if (title!=null) {
            doc.add(Field.Keyword("title", title));
            }
            // add the html title tag to the keywords
            String body = getBody(rawDoc);
            SynopsisUtil.addString(body, doc);
            // now get the meta data and add some fields for that.
            Field[] metaList = getMeta(rawDoc);
            if (metaList != null) {
                for (int lauf = 0; lauf < metaList.length; lauf++) {
                    doc.add(metaList[lauf]);
                }
            }
        } catch (FileNotFoundException fnfE) {
            System.out.println("The file: " + f.getName()
                    + "  caused an filenotfound Excetion "
                    + fnfE.getMessage());
            log.println("The file: " + f.getName()
                    + "  caused an filenotfound Excetion "
                    + fnfE.getMessage());
            doc= null;
        } catch (UnsupportedEncodingException ueE) {
            System.out.println("The file: " + f.getName()
                    + " caused an unsupported encoding" + ueE.getMessage());
            log.println("The file: " + f.getName()
                    + " caused an unsupported encoding" + ueE.getMessage());
            doc= null;
        }

    }
    /**
     * Gets the fields from JTidy for the HTML 
     * @param rawDoc Element is a node in a document (w3c XML doc.)
     * @return Lucene Fields one for each meta data info.
     */
     
    private Field[] getMeta(Element rawDoc) {
        Field[] meta = null;
        Field[] tmp = null;
        List marray = new ArrayList();
        tmp = getFields(rawDoc, "meta");
        if (tmp != null) {
            for (int lauf = 0; lauf < tmp.length; lauf++) {
                marray.add(tmp[lauf]);
            }
        }
        tmp = getFields(rawDoc, "META");
        if (tmp != null) {
            for (int lauf = 0; lauf < tmp.length; lauf++) {
                marray.add(tmp[lauf]);
            }
        }
        tmp = getFields(rawDoc, "Meta");
        if (tmp != null) {
            for (int lauf = 0; lauf < tmp.length; lauf++) {
                marray.add(tmp[lauf]);
            }
        }
        if (!(marray.isEmpty())) {
            meta = (Field[]) marray.toArray(new Field[0]);
        }
        return meta;
    }
    /**
     * This is used as part of JTidy
     * @param rawDocs is a node in a document (w3c XML doc.) 
     * @param tgName the name of the element
     * @return Lucene field array
     */
    private Field[] getFields(Element rawDoc, String tgName) {
        Field[] data = null;
        Field tmpFld;
        List fldArray = new ArrayList();
        NodeList children = rawDoc.getElementsByTagName(tgName);
        if (children.getLength() > 0) {
            for (int lauf = 0; lauf < children.getLength(); lauf++) {
                NamedNodeMap underMeta = children.item(lauf).getAttributes();

                org.w3c.dom.Node name = underMeta.getNamedItem("name");
                org.w3c.dom.Node content = underMeta.getNamedItem("content");
                if ((name != null) && (content != null)) {
                    tmpFld = Field.Keyword(name.getNodeValue(), content
                            .getNodeValue());
                    fldArray.add(tmpFld);
                }
            }
        }
        if (!(fldArray.isEmpty())) {
            data = (Field[]) fldArray.toArray(new Field[0]);
        }
        return data;
    }
    /**
     * Gets the body text of the HTML document.
     * 
     * @rawDoc the DOM Element to extract body Node from
     * @return the body text
     */
    private String getBody(Element rawDoc) {
        if (rawDoc == null) {
            return null;
        }

        String body = "";
        NodeList children = rawDoc.getElementsByTagName("body");
        if (children.getLength() > 0) {
            body = getText(children.item(0));
        }
        return body;
    }
    /**
     * Extracts text from the DOM node.
     * For HTML 
     * @param node
     *            a DOM node
     * @return the text value of the node
     */
    private String getText(org.w3c.dom.Node node) {
        NodeList children = node.getChildNodes();
        StringBuffer sb = new StringBuffer();
        for (int i = 0; i < children.getLength(); i++) {
            org.w3c.dom.Node child = children.item(i);
            switch (child.getNodeType()) {
            case org.w3c.dom.Node.ELEMENT_NODE:
                sb.append(getText(child));
                sb.append(" ");
                break;
            case org.w3c.dom.Node.TEXT_NODE:
                sb.append(((Text) child).getData());
                break;
            }
        }
        return sb.toString();
    }
    /**
     * Gets the title text of the HTML document.
     * 
     * @rawDoc the DOM Element to extract title Node from
     * @return the title text
     */
    private  String getTitle(Element rawDoc) {
        if (rawDoc == null) {
            return null;
        }
        String title = "";
        NodeList children = rawDoc.getElementsByTagName("title");
        if (children.getLength() > 0) {
            Element titleElement = ((Element) children.item(0));
            Text text = (Text) titleElement.getFirstChild();
            if (text != null) {
                title = text.getData();
            }
        }

        return title;
    }
    public String toString(){
    	String name = this.getClass().getName();
    	return name + " Reads HTML files. Uses JTidy to convert to XML conform file and then reads the XML via DOM.";
    	
    }

}
