/*
 * Created on Sep 1, 2005
 * This reader is for pdf using PDF Box
 * Copyright 2005 Tom Louton; Berlin, Germany, http://www.theloutons.com/photoalbumtom.htm 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */
package com.theloutons.search.specialreaders;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.StringWriter;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.pdfbox.pdmodel.PDDocument;
import org.pdfbox.searchengine.lucene.LucenePDFDocument;
import org.pdfbox.util.PDFTextStripper;

import com.theloutons.search.utils.SynopsisUtil;

/**
 * @author Tom Louton
 *
 */
public class PDFReader implements DocumentAnalyze {
    private PrintWriter log;
    private Document doc = null;
    

    /* (non-Javadoc)
     * @see de.schering.ci.search.specialreaders.DocumentAnalyze#getDocument()
     */
    public Document getDocument() {
        return this.doc;
    }

    /* (non-Javadoc)
     * @see de.schering.ci.search.specialreaders.DocumentAnalyze#setFile(java.io.File, java.io.PrintWriter)
     */
    public void setFile(File f, PrintWriter log) {
        doc= new Document();
        try {
            doc = LucenePDFDocument.getDocument(f);
            /*
             * here we get the text again to summarize
             */

            PDDocument pdfDocument = null;
            InputStream is = new FileInputStream(f);
            pdfDocument = PDDocument.load(is);

            StringWriter writer = new StringWriter();
            PDFTextStripper stripper = new PDFTextStripper();
            stripper.writeText(pdfDocument, writer);
            String synop=SynopsisUtil.getSynopsis(writer
                    .getBuffer().toString());
            if (synop!=null){
            doc.add(Field.Keyword("synopsis",synop ));
            }
            pdfDocument.close();
            doc.add(Field.Keyword("docType", "pdf"));
        } catch (IOException ioE) {
            System.out.println("The file: " + f.getName()
                    + " caused an io excepton " + ioE.getMessage());
            log.println("The file: " + f.getName()
                    + " caused an io excepton " + ioE.getMessage());
            doc= null;
        }


    }
    public String toString(){
    	String name = this.getClass().getName();
    	return name + " Reads Adobe pdf format. Usese a package called PDF Box (www.pdfbox.org).";
    	
    }

  
}
