/*
 * Created on Sep 1, 2005
 * Copyright 2005 Tom Louton; Berlin, Germany, http://www.theloutons.com/photoalbumtom.htm 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.theloutons.search.specialreaders;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.PrintWriter;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.poi.hslf.extractor.PowerPointExtractor;

import com.theloutons.search.utils.SynopsisUtil;

/**
 * @author Tom Louton
 *
 */
public class PPTReader implements DocumentAnalyze {
    private PrintWriter log;
    private Document doc=null;

    /**
     * 
     */
    public PPTReader() {
        super();
    }

    /* (non-Javadoc)
     * @see de.schering.ci.search.specialreaders.DocumentAnalyze#getDocument()
     */
    public Document getDocument() {
        return this.doc;
    }

    /* (non-Javadoc)
     * @see de.schering.ci.search.specialreaders.DocumentAnalyze#setFile(java.io.File, java.io.PrintWriter)
     */
    public void setFile(File f, PrintWriter log) {
        this.log=log;
        this.doc= new Document();
        this.doc.add(Field.Keyword("docType", "ppt"));
         
        try {
              FileInputStream fIS = new FileInputStream(f);
              PowerPointExtractor ppE = new PowerPointExtractor(fIS);
                String docTxt =ppE.getText();
                this.doc = SynopsisUtil.addString(docTxt, this.doc);
                String synop =SynopsisUtil.getSynopsis(docTxt);
                if (synop!=null){
                doc.add(Field.Keyword("synopsis", synop));
                }                	
                
                ppE.close();
                
            }
            catch (IOException ioE) {
                log.println(ioE.getMessage());
                doc= null;
            }
            /*
            catch (ArrayIndexOutOfBoundsException aiE){
                System.out.println("ArrayIndex bad");
                return null;
            }
            */

    }
    /**
     * returns a description
     */
    public String toString(){
    	String name = this.getClass().getName();
    	return name + " Reads Power Point (ppt) files. Uses the Apache POI tools to render the ppt file.";
    	
    }

}
