/*
 * Created on Sep 1, 2005
 * Copyright 2005 Tom Louton; Berlin, Germany, http://www.theloutons.com/photoalbumtom.htm 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */
package com.theloutons.search.specialreaders;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.StringReader;

import javax.swing.text.BadLocationException;
import javax.swing.text.StyledDocument;
import javax.swing.text.rtf.RTFEditorKit;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;

import com.theloutons.search.utils.SynopsisUtil;
/**
 * @author Tom Louton
 *
 * 
 */
public class RTFReader implements DocumentAnalyze {
    
private Document doc=null;
private PrintWriter log;
    /**
     * 
     */
    public RTFReader() {
        super();
    }

    /* (non-Javadoc)
     * @see de.schering.ci.search.specialreaders.DocumentAnalyze#getDocument()
     */
    public Document getDocument() {
        return this.doc;
    }

    /* (non-Javadoc)
     * @see de.schering.ci.search.specialreaders.DocumentAnalyze#setFile(java.io.File, java.io.PrintWriter)
     */
    public void setFile(File f, PrintWriter log) {
        this.log=log;
        doc= new Document();
        doc.add(Field.Keyword("docType", "rtf"));
        try {
            FileInputStream stream = new FileInputStream(f);
            RTFEditorKit kit = new RTFEditorKit();
            StyledDocument rtfDoc = (StyledDocument) kit
                    .createDefaultDocument();
            kit.read(stream, rtfDoc, 0);
            String docTxt = rtfDoc.getText(0, rtfDoc.getLength());
            if (docTxt == null) {
                doc= null;
                log.println("the retrieved text was null? ");
                return;
            }
            Reader strReader = new StringReader(docTxt);
            doc.add(Field.Text("contents", strReader));
            /*
             * now get a summary, the first 500 words.
             */
            int lng = docTxt.length();
            int endSum = 500;
            if (lng < 500) {
                endSum = lng;
            }
            doc.add(Field.Keyword("summary", docTxt.substring(0, endSum)));
            //  doc.add(Field.Keyword("synopsis", getSynopsis(docTxt)));
            String synop =  SynopsisUtil.getSynopsis(docTxt);
            if (synop != null) {
                doc.add(Field.Keyword("synopsis",synop));
            } 

        } catch (FileNotFoundException fnfE) {
            System.out.println("file not found " + fnfE.getMessage());
            log.println("file not found " + fnfE.getMessage());
            doc= null;
        } catch (BadLocationException blE) {
            System.out.println("Bad Location " + blE.getLocalizedMessage());
            log.println("Bad Location " + blE.getLocalizedMessage());
            doc= null;
        } catch (IOException ioE) {
            System.out.println("IO E "+ioE.getMessage());
            log.println("IO E "+ioE.getMessage());
            doc= null;
        }

    }
    /**
     * returns a description
     */
    public String toString(){
    	String name = this.getClass().getName();
    	return name + " Reads Rich Text Format files. Uses the java swing rtf package (javax.swing.text.rtf).";
    	
    }

}
