/*
 * Created on Sep 1, 2005
 * Copyright 2005 Tom Louton; Berlin, Germany, http://www.theloutons.com/photoalbumtom.htm 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.theloutons.search.specialreaders;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;

import com.theloutons.search.utils.SynopsisUtil;


/**
 * @author Tom Louton
 *
 *
 */
public class TXTReader implements DocumentAnalyze {
    private Document doc;
    private PrintWriter log;

    /**
     * 
     */
    public TXTReader() {
        super();
    }

    /* (non-Javadoc)
     * @see de.schering.ci.search.specialreaders.DocumentAnalyze#getDocument()
     */
    public Document getDocument() {
        return this.doc;
    }

    /* (non-Javadoc)
     * @see de.schering.ci.search.specialreaders.DocumentAnalyze#setFile(java.io.File, java.io.PrintWriter)
     */
    public void setFile(File f, PrintWriter log) {
        this.log=log;
        this.doc= new Document();
        this.doc.add(Field.Keyword("docType", "txt"));
        this.doc = dealwithText(f, doc);

    }
    /**
     * 
     * @param f File the object of all this.
     * @param doc a lucene Document
     * @return the return document with added info
     */
    private Document dealwithText(File f,
            org.apache.lucene.document.Document doc) {

        try {
            FileInputStream fIS = new FileInputStream(f);
            InputStreamReader isRead = new InputStreamReader(fIS, "UTF-8");
            BufferedReader bR = new BufferedReader(isRead);
            String docTxt = "";
            String line;
            while ((line = bR.readLine()) != null) {
                docTxt = docTxt + line;
            }
            doc = SynopsisUtil.addString(docTxt, doc);
        } catch (UnsupportedEncodingException ueE) {
            System.out.println("Unsupported encoding" + f.getName());
            log.println("Unsupported encoding" + f.getName());
            return null;
        } catch (IOException ioE) {
            System.out.println("IO Exception " + f.getName());
            log.println("IO Exception " + f.getName());
            return null;
        }
        return doc;
    }
    /**
     * returns a description for class
     */
    public String toString(){
    	String name = this.getClass().getName();
    	return name + " Reads any sort of text file.";
    	
    }

}
