/*
 * Created on Sep 1, 2005
 * Copyright 2005 Tom Louton; Berlin, Germany, http://www.theloutons.com/photoalbumtom.htm 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */
package com.theloutons.search.specialreaders;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;

import org.apache.lucene.document.Document;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;

import com.theloutons.search.utils.SynopsisUtil;

/**
 * @author Tom Louton
 *
 */
public class XLSReader implements DocumentAnalyze {
private Document doc=null;
private PrintWriter log;

    /**
     * 
     */
    public XLSReader() {
        super();
    }

    /* (non-Javadoc)
     * @see de.schering.ci.search.specialreaders.DocumentAnalyze#getDocument()
     */
    public Document getDocument() {
        // TODO Auto-generated method stub
        return this.doc;
    }

    /* (non-Javadoc)
     * @see de.schering.ci.search.specialreaders.DocumentAnalyze#setFile(java.io.File, java.io.PrintWriter)
     */
    public void setFile(File f, PrintWriter log) {
        this.log=log;
        if (doc==null){
        doc = new Document();
        }
        this.doc=indexExcel(f,doc);
    }
    /**
     * This extracts the text (only text cells) and adds them along
     * with the titles of the work sheets
     * @param f reference to Excel file
     * @param doc the lucene docuemnt to which to add the 
     * text found
     * @return the lucene docuemnt updated.
     */
    private Document indexExcel(File f, Document doc){
        try {
            InputStream inputStr = new FileInputStream(f);
            POIFSFileSystem fs = new POIFSFileSystem(inputStr);
            HSSFWorkbook workbook = new HSSFWorkbook(fs);
            int shNr = workbook.getNumberOfSheets();
            String textCont = "";
            HSSFSheet blat;
            for (int lauf = 0; lauf < shNr; lauf++) {
                blat = workbook.getSheetAt(lauf);
                textCont= textCont +" "+workbook.getSheetName(lauf);
                if (!(blat == null)) {
                    int frstrw = blat.getFirstRowNum();
                    int lstrw = blat.getLastRowNum();
                    for (int btw = frstrw; btw < lstrw; btw++) {
                        textCont = textCont + getRowText(blat.getRow(btw));
                    }
                }
            }
            if (textCont!=null) {
            doc = SynopsisUtil.addString(textCont, doc);
           // System.out.println(textCont);
            }
        } catch (FileNotFoundException fnfE) {
            System.out.println("fileNotfound Ex "+fnfE.getMessage());
            log.println("fileNotfound Ex "+fnfE.getMessage());
            return null;
        } catch (IOException ioE) {
            System.out.println("IO E "+ioE.getMessage());
            log.println("IO E "+ioE.getMessage());
            return null;
        }
        return doc;
    }
    /**
     * This is a short support for the excel extraction 
     * @param row  the row from which to extract the text 
     * from all cells.
     * @return the accumulated text.
     */
    private String getRowText(HSSFRow row) {
        String rt = "";
        if (row == null) {
            return rt;
        } else {
            short frstcl = row.getFirstCellNum();
            short lstcl = row.getLastCellNum();
            for (short cls = frstcl; cls < lstcl; cls++) {
                if (!(row.getCell(cls) == null)) {
                    int ctype = row.getCell(cls).getCellType();
                    switch (ctype) {
                    case HSSFCell.CELL_TYPE_NUMERIC:
                        //  rt = rt + row.getCell(cls).getNumericCellValue();
                        break;
                    case HSSFCell.CELL_TYPE_STRING:
                        rt = rt + " " + row.getCell(cls).getStringCellValue();
                        break;
                    default:
                        break;
                    }
                }
            }
        }
        return rt;
    }
    /**
     * returns a description for class
     */
    public String toString(){
    	String name = this.getClass().getName();
    	return name + " Reads Excel files (xls) and returns the cells which contain text.";
    	
    }
}
