/*
 * Created on Aug 29, 2005
 *
 * Copyright 2005 Tom Louton; Berlin, Germany, http://www.theloutons.com/photoalbumtom.htm 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 */
package com.theloutons.search.specialreaders;

import org.apache.lucene.document.Field;

import java.io.FileInputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.StringReader;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.xml.sax.SAXException;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author Tom Louton
 * Need to create the Lucene Document and the deal with it and
 * the reading of the special file.
 */
public class XMLReader implements DocumentAnalyze {

    private org.apache.lucene.document.Document doc=null;
    private String docText = "";
    private PrintWriter log;

    /**
     * use this, as it most likely will be instantiated by
     *  
     */
    public XMLReader() {
        super();
        this.doc = new org.apache.lucene.document.Document();
    }

    /**
     *  
     */
    public XMLReader(String fileName) {
        super();
        this.doc = new org.apache.lucene.document.Document();

        try {
            FileInputStream fiS = new FileInputStream(new File(fileName));
        } catch (FileNotFoundException fnfE) {
            this.doc = null;
        }
    }

    public XMLReader(FileInputStream fiS) {
        super();
        this.doc = new org.apache.lucene.document.Document();
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.schering.ci.search.interfaces.DocumentAnalyze#getDocument()
     */
    public org.apache.lucene.document.Document getDocument() {
        return this.doc;
    }

    public void setFile(File f,PrintWriter log) {
        this.log=log;
        try {
            FileInputStream fiS = new FileInputStream(f);
            readXMLFile(fiS);
        } catch (FileNotFoundException fnfE) {
            log.println("FileNotFoundException: "+fnfE.getMessage());
            this.doc = null;
        }
    }

    private void readXMLFile(FileInputStream fiS) {
        if(doc ==null){
            this.doc = new org.apache.lucene.document.Document();
           }
        Node docNd = null;
        try {
            DocumentBuilderFactory factory = DocumentBuilderFactory
                    .newInstance();
            DocumentBuilder builder = factory.newDocumentBuilder();
            docNd = builder.parse(fiS);
            readNode(docNd);
            Reader strReader = new StringReader(docText);
            doc.add(Field.Text("contents", strReader));

        } catch (ParserConfigurationException pcE) {
            System.out.println("Parser Config Exception " + pcE.getMessage());
            log.println("Parser Config Exception " + pcE.getMessage());
            this.doc = null;
        } catch (IOException ioE) {
            System.out.println("IO Exception " + ioE.getMessage());
            log.println("IO Exception " + ioE.getMessage());
            this.doc = null;
        } catch (SAXException saxE) {
            System.out.println("SAX Exception " + saxE.getMessage());
            log.println("SAX Exception " + saxE.getMessage());
            this.doc = null;
        }
        if (!(docNd == null)) {
            NodeList nl = docNd.getChildNodes();
        }
    }

    private void readNode(Node nd) {

        NodeList ndl = null;
        switch (nd.getNodeType()) {
        case Node.ATTRIBUTE_NODE:
            // for the moment skip
            break;
        case Node.CDATA_SECTION_NODE:
            this.addString(nd.getNodeValue());
            break;
        case Node.COMMENT_NODE:
            this.addString(nd.getNodeValue());
            break;
        case Node.DOCUMENT_FRAGMENT_NODE:
            ndl = nd.getChildNodes();
            for (int lauf = 0; lauf < ndl.getLength(); lauf++) {
                readNode(ndl.item(lauf));
            }
            break;
        case Node.DOCUMENT_NODE:
            ndl = nd.getChildNodes();
            for (int lauf = 0; lauf < ndl.getLength(); lauf++) {
                readNode(ndl.item(lauf));
            }
            break;
        case Node.DOCUMENT_TYPE_NODE:
            ndl = nd.getChildNodes();
            for (int lauf = 0; lauf < ndl.getLength(); lauf++) {
                readNode(ndl.item(lauf));
            }
            break;
        case Node.ELEMENT_NODE:
            ndl = nd.getChildNodes();
            for (int lauf = 0; lauf < ndl.getLength(); lauf++) {
                readNode(ndl.item(lauf));
            }

            break;
        case Node.ENTITY_NODE:

            break;
        case Node.ENTITY_REFERENCE_NODE:

            break;
        case Node.NOTATION_NODE:

            break;
        case Node.PROCESSING_INSTRUCTION_NODE:

            break;
        case Node.TEXT_NODE:
            this.addString(nd.getNodeValue());
            break;

        }
    }

    private void addString(String text) {
        this.docText = this.docText + text;
    }
    public String toString(){
    	String name = this.getClass().getName();
    	return name + " Reads any XML file. Just gets the text nodes.";    	
    }

}
