/*
 * Copyright 2005 Tom Louton; Berlin, Germany, http://www.theloutons.com/photoalbumtom.htm 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.theloutons.search.userinterfaces;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.HeadlessException;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JRootPane;
import javax.swing.JTextArea;
import javax.swing.WindowConstants;

import com.theloutons.search.utils.FrameVersionInfo;
/**
 * 
 * @author Tom Louton
 *
 */
public class ConfigEditor extends JFrame {
    final Container contentPane = getContentPane();

    private boolean save = false;

    private JTextArea edit;
    private String mozilla= "C:/Program Files/Internet Explorer/iexplore.exe \"";
 
 
    /**
     * This is a trivial config editor
     * 
     * @param title
     *            Title of the editor
     * @param init
     *            the contents of the initial screen -- the configuration
     * @throws HeadlessException
     */
    public ConfigEditor(String title, String init,String mozilla) throws HeadlessException {
        super(title);
        this.mozilla=mozilla;
        setupWindow(init);

     }

  

    /**
     * Set up the initial screen
     * 
     * @param init
     */
    private void setupWindow(String init) {
        addMenues();
        JRootPane jrp = getRootPane();
        jrp.setPreferredSize(new Dimension(900, 600));
        edit = new JTextArea(init);
        contentPane.add(edit);
    }

    /**
     * Gets the contents of the editor.
     * 
     * @return the string containing the configuration
     */
    public String getText() {
        return edit.getText();
    }

    /**
     * adds the menus
     *  
     */
    private void addMenues() {
        // put in a menu bar

        JMenuBar menuBar = new JMenuBar();
        JMenu menu = new JMenu("Menu");
        JMenuItem save = new JMenuItem("Save");
        save.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setToSave();
                dispose();
            }
        });
        JMenuItem discard = new JMenuItem("Discard");
        discard.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setToDiscard();
                dispose();
                //System.exit(0);
            }
        });

        JMenuItem itemEnd = new JMenuItem("End Application NOW");
        itemEnd.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                setToDiscard();
                dispose();
            }
        });
        menu.add(save);
        menu.add(discard);
        menu.add(itemEnd);
        menuBar.add(menu);

        
        // put in a help menu bar
        JMenu helpMenu = new JMenu("Help");
        JMenuItem itemHelp = new JMenuItem("About private search");
        itemHelp.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JFrame info = new  FrameVersionInfo(mozilla);
                info.setLocation(30, 30);
                info.pack();
                info.setVisible(true);

                info.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

                info.addWindowListener(new WindowAdapter() {
                    public void windowClosed(WindowEvent e) {
                       System.out.println("closed window");
                    }
                });

           }
        });
        helpMenu.add(itemHelp);
        menuBar.add(helpMenu);
        setJMenuBar(menuBar);

    }

    /**
     * Marks whether or not the configuration should be written to the file.
     * 
     * @return
     */
    public boolean saveOrNot() {
        return this.save;
    }

    /**
     * sets the marker to save.
     *  
     */
    private void setToSave() {
        this.save = true;
    }

    /**
     * Sets marker to not save.
     *  
     */
    private void setToDiscard() {
        this.save = false;
    }

}