/*
 * Created on Aug 4, 2005
 * This is a user interface combined with an indexer for Lucene
 * Copyright 2005 Tom Louton, Berlin, Germany, http://www.theloutons.com/photoalbumtom.htm 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.theloutons.search.userinterfaces;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.image.FilteredImageSource;
import java.awt.image.ImageFilter;
import java.awt.image.ReplicateScaleFilter;
import java.awt.Button;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Container;
import java.awt.Component;
import java.awt.Font;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.Insets;

// import java.awt.event.*;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringReader;

import java.util.ArrayList;
import java.util.Date;
import java.util.Enumeration;
import java.util.List;
import java.util.Properties;

import javax.imageio.ImageIO;
import javax.swing.Box;
import javax.swing.border.Border;
import javax.swing.BorderFactory;
import javax.swing.filechooser.FileSystemView;
// import java.awt.BorderLayout;
// import javax.swing.filechooser.FileFilter;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JRootPane;
import javax.swing.JSeparator;
// import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.WindowConstants;
// import javax.swing.JScrollPane;
// import javax.swing.colorchooser.AbstractColorChooserPanel;

import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.IndexWriter;
import org.w3c.dom.Node;

import com.theloutons.search.specialreaders.DocumentAnalyze;
import com.theloutons.search.utils.*;

/**
 * @author Tom Louton This class needes to called up from a starter file, e.g.,
 *         JFrame app = new UserIndexerInterface("Title"); This is a class of
 *         course, and the application is an interface
 *  
 */
public class UserIndexerInterface extends JFrame {

    final Container contentPane = getContentPane();

    // the struts are used to put lines between the lables and boxes etc.
    private Component myStrut13 = Box.createVerticalStrut(13);

    private Component myStrut10 = Box.createVerticalStrut(10);

    private Component myStrut3 = Box.createVerticalStrut(3);

    // I have made these global so that I do not need to pass them between
    // methods, and to keep from creating lots of instances.
    final JButton indexButton = new JButton(
            "Choose directory in which to write the index");

    final JButton sourceButton = new JButton(
            "Choose the direcory to be indexed");

    final ButtonwithToggle okButton = new ButtonwithToggle("Index now!");

    final JButton killButton = new JButton("Done");

    final JButton stopButton = new JButton("Exit");

    final JButton getHelp = new JButton("Documents");

    final JButton getConfig = new JButton("Configure");

    final ButtonwithToggle viewLog = new ButtonwithToggle("View Log");

    FileSystemView fsV = FileSystemView.getFileSystemView();

    final JFileChooser chooseIndexlocation = new JFileChooser(fsV);

    final JFileChooser chooseSource = new JFileChooser();

    final JTextField targetDir = new JTextField(40);

    final JTextField sourceDir = new JTextField("The source directory", 40);

    final JLabel title = new JLabel("The Indexer Application, based on ");

    final JLabel logo = new JLabel();

    final JLabel warning1 = new JLabel("CAREFUL");

    final JLabel warning2 = new JLabel(
            "The directory chosen for the index will be relieved of its contents.");

    final JLabel warning3 = new JLabel(
            "Everything in it will be erased! BE CAREFUL!");

    final JLabel indexLabel = new JLabel("The directory to write the index.");

    final JLabel exitLabel = new JLabel("Exit Application");

    final JLabel sourceLabel = new JLabel(
            "The directory containing the files to be indexed.");

    final JLabel startLabel = new JLabel("Return to go, do not ...");

    final JLabel chooselb = new JLabel("Choose which type of result");

    final GridBagLayout gbl = new GridBagLayout();

    final GridBagConstraints gbc = new GridBagConstraints();

    File tgFile = null; // the place in which the index is written.

    File srcFile = null; // the directory to be indexed

    PrintWriter log = null; // used for the log

    String textedit = "notepad";

    String mozilla = "C:/Program Files/Internet Explorer/iexplore.exe \"";

    Properties extraFileTypes = new Properties();

    String index;

    String dirToIndex;

    // define a default
    String configPath = System.getProperty("user.dir")
            + "/data/indexer.defaults";

    JRootPane jrp = getRootPane();

    /**
     * 
     * @param title
     *            The window title
     * @param path
     *            path to the index dir. May be null
     * @throws HeadlessException
     */
    public UserIndexerInterface(String title, String path, String indexPath)
            throws HeadlessException {
        super(title);
        this.configPath = path;
        Properties def = new Properties();
        try {
            FileInputStream fiS = new FileInputStream(new File(path));
            def.load(fiS);
        } catch (IOException ioE) {

        }
        setConfig(def);
        setupWindow();
        if (indexPath != null) {
            if ((new File(indexPath)).isDirectory()) {
                if ((new File(indexPath)).list().length > 0)
                         {

                    String msg = "The directory with which you have initialized is not empty"+"\nDo you wish use the configfuration anyways?";
                    // just issue a bit of a warning
                    if (JOptionPane.showConfirmDialog(null, msg, "Continue?",
                            JOptionPane.YES_NO_CANCEL_OPTION,
                            JOptionPane.QUESTION_MESSAGE, null) == JOptionPane.YES_OPTION) {
                        targetDir.setForeground(Color.BLUE);
                        tgFile = new File(indexPath);
                        targetDir.setText(indexPath);
                        okButton.indexKnown();
                    }

                } else {
                    targetDir.setForeground(Color.BLUE);
                    tgFile = new File(indexPath);
                    targetDir.setText(indexPath);
                    okButton.indexKnown();
                }
            }
        }

    }

    /**
     * This just sets up the start window. This application has in effect only
     * one window or widget. Here we prepare, add actions to buttons etc. the
     * method getStartScreen sets up layout etc.
     *  
     */
    private void setupWindow() {
        jrp.setPreferredSize(new Dimension(750, 450));
        jrp.setBorder(BorderFactory
                .createTitledBorder("Move the Mouse to Move Duke"));
        jrp.setBorder(BorderFactory.createLineBorder(Color.black));
        Border paneEdge = BorderFactory.createEmptyBorder(0, 10, 10, 10);
        jrp.setBorder(paneEdge);
        getStartScreen(); // this addes some menu items

        targetDir.setForeground(Color.RED);
        targetDir.setAlignmentX(Button.CENTER_ALIGNMENT);

        sourceDir.setForeground(Color.RED);
        sourceDir.setAlignmentX(Button.CENTER_ALIGNMENT);

        // now another action This is the file choosing button on
        // second screen sourceButton is for the target (souce) dirctory
        sourceButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                int state = chooseIndexlocation.showOpenDialog(null);
                File file = chooseIndexlocation.getSelectedFile();
                String filename = getFilename(file);

                if (filename != null && state == JFileChooser.APPROVE_OPTION) {
                    // here we set up something
                    // targetDir.setBackground(Color.BLUE);
                    sourceDir.setForeground(Color.BLUE);
                    okButton.searchSet();
                    sourceDir.setText(filename);
                } else {
                    if (state == JFileChooser.CANCEL_OPTION) {
                        JOptionPane.showMessageDialog(null, "Canceled");
                    }
                    if (state == JFileChooser.ERROR_OPTION) {
                        JOptionPane.showMessageDialog(null, "Error!");
                    }
                }
            }

        }); // action listner

        /**
         * now another action This is the file choosing button on second screen
         */
        indexButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                int state = chooseIndexlocation.showOpenDialog(null);
                tgFile = chooseIndexlocation.getSelectedFile();
                String filename = getFilename(tgFile);

                if (filename != null && state == JFileChooser.APPROVE_OPTION) {
                    // here we set up something
                   if (tgFile.list().length==0){
                    targetDir.setForeground(Color.BLUE);
                    okButton.indexKnown();
                    targetDir.setText(filename);
                   }
                   else {
                   //again here just issue warning.  
                       String msg= "The directory for the index is not empty. \n" +
                       		" Overwrite it?";
                       if (JOptionPane.showConfirmDialog(null, msg, "Continue?",
                               JOptionPane.YES_NO_CANCEL_OPTION,
                               JOptionPane.QUESTION_MESSAGE, null) == JOptionPane.YES_OPTION) {
                           targetDir.setForeground(Color.BLUE);
                           targetDir.setText(filename);
                           okButton.indexKnown();
                       }
                   }
                } else {
                    if (state == JFileChooser.CANCEL_OPTION) {
                        JOptionPane.showMessageDialog(null, "Canceled");
                    }
                    if (state == JFileChooser.ERROR_OPTION) {
                        JOptionPane.showMessageDialog(null, "Error!");
                    }
                }
            }
        }); // action listner
        /**
         * This is the most action packed action listner We check that the
         * directories are ok -- see below If the user has entered a directory
         * which does not exist we create it -- only one level tho. Exceptions
         * (at least those I could foresee) are captured and message boxes try
         * to explain to the user how to correct his behavior Then if the source
         * and target are ok, then start the actual indexing, via the function
         * "index(File,File,String)"
         */
        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String sourceName = sourceDir.getText();
                String targetName = targetDir.getText();
                String tgPath = null;
                String srcPath = null;
                File tgFile = new File(targetName);
                File srcFile = new File(sourceName);
                try {
                    tgPath = tgFile.getCanonicalPath();
                    srcPath = srcFile.getCanonicalPath();
                } catch (IOException ioE) {
                    String msg = "An Exception occured." + ioE.getMessage();
                    JOptionPane.showMessageDialog(null, msg
                            + "\nFix this please!");

                }
                if (!tgFile.isDirectory()) {
                    if (!tgFile.mkdir()) {
                        boolean sistgDir = tgFile.mkdirs();
                    }
                    if (!tgFile.isDirectory()) {

                        String msg = "" + tgPath
                                + " does not exist and could not be created!";
                        JOptionPane.showMessageDialog(null, msg
                                + "\nFix this please!");
                        return;
                    }

                } 
                
                if (srcFile.isDirectory()) {
                    int tgLn = tgPath.length();
                    String comp = null;

                    if (tgLn < srcPath.length()) {
                        comp = srcPath.substring(0, tgLn - 1);
                    } else {
                        comp = srcPath;
                    }
                    if (tgPath.startsWith(comp)) {
                        // this is to make sure the index is not being
                        // searched and indexed
                        // we must return to the window
                        JOptionPane
                                .showMessageDialog(null,
                                        "This won't work, the index dir would be indexed. Fix this!");
                        return;
                    }

                    String msg = "index dir: " + tgPath + "\nAnd source dir:"
                            + srcPath;

                    switch (JOptionPane.showConfirmDialog(null, msg,
                            "Continue?", JOptionPane.YES_NO_CANCEL_OPTION,
                            JOptionPane.QUESTION_MESSAGE, null)) {
                    case JOptionPane.YES_OPTION: {
                        // System.out.println("continue and do it");
                        try {
                            String logFileName = tgFile.getAbsolutePath()
                                    + System.getProperty("file.separator")
                                    + "log.txt";
                            int fileNo = index(tgFile, srcFile, logFileName);
                            viewLog.indexKnown();
                            JOptionPane.showMessageDialog(null,
                                    "The index has been created and " + fileNo
                                            + " files were indexed.");

                        } catch (IOException ioE) {
                            String iomsg = ioE.getMessage();
                            JOptionPane
                                    .showMessageDialog(null,
                                            "The index method had a problem:  "
                                                    + iomsg);
                            break;
                        }
                        break;
                    }
                    case JOptionPane.NO_OPTION: {
                        System.out.println("no option");
                        break;
                    }
                    case JOptionPane.CANCEL_OPTION: {
                        break;
                    }
                    default: {
                    }
                    } // end of switch

                } else {

                    JOptionPane
                            .showMessageDialog(null,
                                    "The source directory of the files to index does not exist!");
                }

            }
        }); // action listner
        /**
         * just quits
         */
        stopButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }

        });
        /**
         * now the view documents button
         */
        getHelp.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Runtime view = Runtime.getRuntime();

                try {
                    Process outHTM = view.exec(mozilla
                            + System.getProperty("user.dir")
                            + "/data/indexerandsearcherusage.html#indexing"
                            + "\"");
                } catch (IOException ioE) {
                    JOptionPane.showMessageDialog(null,
                            "There was a problem starting the exporere "
                                    + ioE.getMessage() + " Sorry.");
                    ioE.getStackTrace();
                }
            }

        });
        /**
         *  
         */
        getConfig.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String initialData = getConfigData();
                final ConfigEditor configEdit = new ConfigEditor(
                        "The Configuration Editor", initialData,mozilla);
                configEdit.setLocation(30, 30);
                configEdit.pack();
                configEdit.setVisible(true);

                configEdit
                        .setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
                configEdit.addWindowListener(new WindowAdapter() {
                    public void windowClosed(WindowEvent e) {
                        boolean save = configEdit.saveOrNot();
                        if (save) {
                            // then write to file.
                            String text = configEdit.getText();
                            saveConfigData(text);
                        }
                    }

                });

            }

        });

        /**
         * now the view log button
         */
        viewLog.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Runtime view = Runtime.getRuntime();
                // we can use mozilla here too. Notepad not necessary.
                try {
                    Process outHTM = view.exec(textedit
                            + tgFile.getCanonicalPath() + "/log.txt" + "\"");
                } catch (IOException ioE) {
                    JOptionPane.showMessageDialog(null,
                            "There was a problem starting NOTEPAD "
                                    + ioE.getMessage() + " Sorry.");
                    ioE.getStackTrace();
                }
            }

        });
    }

    /**
     * This sets up the first entry screen.
     */
    private void getStartScreen() {

        // add action listener to see if it helps
        // button to move to second screen
        // reset the DOM for the config data

        // put in a menu bar
        JMenuBar menuBar = new JMenuBar();
        JMenu menu = new JMenu("Menu");
        JMenuItem itemEnd = new JMenuItem("End Application NOW");
        itemEnd.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }
        });
        menu.add(itemEnd);
        menuBar.add(menu);

        // put in a help menu bar
        JMenu helpMenu = new JMenu("Help");
        JMenuItem itemHelp = new JMenuItem("About private search");
        itemHelp.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JFrame info = new  FrameVersionInfo(mozilla);
                info.setLocation(30, 30);
                info.pack();
                info.setVisible(true);

                info.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

                info.addWindowListener(new WindowAdapter() {
                    public void windowClosed(WindowEvent e) {
                       System.out.println("closed window");
                    }
                });
            }
        });
        JMenuItem itemFileTypes = new JMenuItem("File types indexed");
        itemFileTypes.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JOptionPane
                        .showMessageDialog(
                                null,
                                "The following are the types of files indexed\n"
                                        + "blk, csv, doc, html, pdf, ppt, rtf, txt, xls");
            }
        });
        helpMenu.add(itemFileTypes);
        helpMenu.add(itemHelp);
        menuBar.add(helpMenu);
        setJMenuBar(menuBar);

        /**
         * Component[] contList = contentPane.getComponents(); int compCount =
         * contList.length; for (int lauf = 0; lauf < compCount; lauf++) {
         * contentPane.remove(contList[lauf]); }
         */

        chooseIndexlocation.setAcceptAllFileFilterUsed(false);
        chooseIndexlocation.setMultiSelectionEnabled(false);
        chooseIndexlocation.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

        contentPane.repaint();
        contentPane.validate();
        contentPane.setLayout(gbl);

        gbc.anchor = GridBagConstraints.CENTER;
        gbc.gridwidth = GridBagConstraints.RELATIVE;
        gbc.insets = new Insets(30, 0, 0, 0);
        logo.setIcon(setLogo("data/tkl.png", 40, 30));
        title.setFont(new Font("Times New Roman", Font.BOLD, 20));
        title.setIcon(setLogo("data/tkl.png", 40, 30));
        contentPane.add(title, gbc);

        gbc.anchor = GridBagConstraints.EAST;
        // gbc.gridwidth = GridBagConstraints.RELATIVE;
        // contentPane.add(logo, gbc);
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        JLabel lucene = new JLabel(setLogo("data/lucene.gif", 150, 23));
        contentPane.add(lucene, gbc);

        // add a waringing
        gbc.anchor = GridBagConstraints.WEST;
        warning1.setFont(new Font("serif", Font.BOLD, 18));
        warning1.setForeground(Color.RED);
        warning2.setFont(new Font("serif", Font.BOLD, 14));
        warning2.setForeground(Color.RED);
        warning3.setFont(new Font("serif", Font.BOLD, 14));
        warning3.setForeground(Color.RED);
        gbc.insets = new Insets(0, 0, 0, 0);
        gbc.gridwidth = GridBagConstraints.RELATIVE;
        contentPane.add(new JSeparator());
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.anchor = GridBagConstraints.CENTER;
        contentPane.add(warning1, gbc);
        contentPane.add(warning2, gbc);
        contentPane.add(warning3, gbc);

        // block for input dir the dir containing the index.
        gbc.insets = new Insets(30, 0, 0, 0);
        gbc.gridwidth = GridBagConstraints.RELATIVE;
        contentPane.add(new JSeparator());
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        contentPane.add(indexLabel, gbc);

        gbc.insets = new Insets(0, 0, 0, 0);
        contentPane.add(targetDir, gbc);

        gbc.insets = new Insets(30, 0, 0, 0);
        contentPane.add(new JSeparator());

        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.insets = new Insets(0, 0, 0, 0);
        contentPane.add(indexButton, gbc);

        // ###########
        // the same for the source directory
        gbc.insets = new Insets(30, 0, 0, 0);
        gbc.gridwidth = GridBagConstraints.RELATIVE;
        contentPane.add(new JSeparator());
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        contentPane.add(sourceLabel, gbc);
        gbc.insets = new Insets(0, 0, 0, 0);
        contentPane.add(sourceDir, gbc);
        contentPane.add(new JSeparator());

        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.insets = new Insets(0, 0, 0, 0);
        contentPane.add(sourceButton, gbc);

        gbc.gridwidth = GridBagConstraints.RELATIVE;
        contentPane.add(okButton, gbc);
        viewLog.searchSet();
        contentPane.add(viewLog, gbc);
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        contentPane.add(getHelp, gbc);
        contentPane.add(getConfig, gbc);
        gbc.anchor = GridBagConstraints.EAST;
        stopButton.setBackground(Color.RED);
        contentPane.add(stopButton, gbc);
    } // end of getStartScreen

    /**
     * this is just a small support to get the path for a file
     * 
     * @param file
     *            the file whose path is needed.
     * @return the path as string
     */
    private String getFilename(File file) {
        if (file != null) {
            return file.getPath();
        }
        return null;
    }

    /**
     * 
     * @param indexDir
     *            location of in
     * @param dataDir
     * @return number of files processed
     * @throws IOException
     */
    private int index(File indexDir, File dataDir, String logFileName)
            throws IOException {
        if (!dataDir.exists() || !dataDir.isDirectory()) {
            throw new IOException(dataDir
                    + " does not exist or is not a directory");
        }
        StopWordMgmt def = new StopWordMgmt();
        String[] stopWords = def.getDefault();

        IndexWriter writer = new IndexWriter(indexDir, new StandardAnalyzer(
                stopWords), true);
        writer.setUseCompoundFile(false);
        // here we set up a log.
        File logFile = new File(logFileName);
        FileWriter logWriter = new FileWriter(logFile);
        log = new PrintWriter(logWriter);
        log.println("Indexing " + indexDir.getAbsolutePath() + "with "
                + dataDir.getAbsolutePath()
                + " as the target directoy for the index.\n");
        long start = new Date().getTime();

        log.println("Started at " + start + " --  measured in java time.\n");
        log.println("The following could not be indexed.\n");

        indexDirectory(writer, dataDir);

        int numIndexed = writer.docCount();

        long end = new Date().getTime();
        log.println("Ended at " + end + "   measured in java time.\n");
        log.println(numIndexed + " files were indexed.");
        log.close();
        writer.optimize();
        writer.close();
        writeIndexConfig(indexDir.getCanonicalPath());
        return numIndexed;
    }

    /**
     * Recursive method that calls itself when it finds a directory
     * 
     * @param writer
     *            Lucene index writer, aimed at the future index directory
     * @param dir
     *            the directory to be recursively searched for files
     * @throws IOException
     *             Should try to cathc this to keep program from colapsing in
     *             mid air
     */
    private void indexDirectory(IndexWriter writer, File dir)
            throws IOException {
        // File[] selected
        File[] files = dir.listFiles();
        // File[] files =selected;
        if (files != null) {
            for (int lauf = 0; lauf < files.length; lauf++) {
                File f = files[lauf];
                if (f.isDirectory()) {
                    indexDirectory(writer, f);
                }

                else {
                    indexFile(writer, f);
                }
            } // if
        }
    }

    /**
     * the method to actually index the file
     */
    private void indexFile(IndexWriter writer, File f) throws IOException {
        if (f.isHidden() || !f.exists() || !f.canRead()) {
            return;
        }
        System.out.println("Indexing " + f.getCanonicalPath());
        Document doc = DocReadOrgainzer.getDocument(f, extraFileTypes, log);
        if (doc != null) {
            System.out.println("Adding " + f.getCanonicalPath());
            writer.addDocument(doc);
        } else {
            log.println(f.getCanonicalPath() + " could not be indexed.\n");
        }
    }

    /**
     * This is just for user interface
     * 
     * @param imName
     *            file name for image
     * @param iw
     *            width of image (px)
     * @param ih
     *            hight of image (px)
     * @return the image as logo
     */
    private Icon setLogo(String imName, int iw, int ih) {
        Image im = null;
        try {
            Image logoIm = ImageIO.read(new File(imName));
            ImageFilter imFil = new ReplicateScaleFilter(iw, ih);
            FilteredImageSource imSour = new FilteredImageSource(logoIm
                    .getSource(), imFil);
            im = createImage(imSour);

        } catch (IOException ioE) {
            ioE.getStackTrace();
            return null;
        }
        return (Icon) new ImageIcon(im);

    }

    /**
     * Allows one to feed info at the moment only "mozilla.call" which contains
     * the browser call e.g. in Linux "firefox" and in our windows C:/Program
     * Files/Internet Explorer/iexplore.exe, the last \" is to start the
     * encapsulation of the file name in quotes
     * 
     * @param def
     *            The config properties
     */
    public void setConfig(Properties def) {
        if (!(def.getProperty("mozilla.call") == null)) {
            mozilla = ((String) def.getProperty("mozilla.call")) + " \"";
        }
        if (!(def.getProperty("textedit.call") == null)) {
            textedit = ((String) def.getProperty("textedit.call")) + " \"";
        }
        Enumeration keys = def.keys();
        int count = 0;
        while (keys.hasMoreElements()) {
            String type = (String) keys.nextElement();
            String sample = "filetype";
            if (type.startsWith(sample)) {
                // make life easy and just chop off the rest which
                // is a suffix for the file maybe comma sep.
                String suffix = type.substring(9);
                String[] ends = getCommaSepParts(type.substring(9));
                for (int lauf = 0; lauf < ends.length; lauf++) {
                    extraFileTypes.setProperty(ends[lauf], def
                            .getProperty(type));
                }
            }
        }

    }

    /**
     * This is for the case that for txt,me,blank read.
     * 
     * @param suffix
     * @return
     */
    private String[] getCommaSepParts(String suffix) {
        String[] rt = null;
        List tmprt = new ArrayList();

        int wo = suffix.indexOf(',');
        int start = 0;
        while ((wo != -1) && (wo < suffix.length())) {
            tmprt.add(suffix.substring(start, wo));
            start = wo + 1;
            wo = suffix.indexOf(',', start + 1);
        }
        if (wo == suffix.length()) {
            tmprt.add("");
        }
        if (wo == -1) {
            tmprt.add(suffix.substring(start));
        }
        rt = (String[]) tmprt.toArray(new String[0]);
        return rt;
    }

    /**
     * 
     * @return
     */
    private String getConfigData() {
        String initialData = "";
        try {
            String line;
            BufferedReader bR = new BufferedReader(new FileReader(
                    this.configPath));
            while ((line = bR.readLine()) != null) {
                initialData = initialData + line + "\n";
            }
            return initialData;
        } catch (IOException ioE) {
            System.out.println("Problem with config data " + ioE.getMessage());
            return initialData;
        }

    }

    /**
     * writes the config file
     * 
     * @param text
     *            the configuration
     */
    private void saveConfigData(String text) {
        try {

            PrintWriter out = new PrintWriter(new FileWriter(new File(
                    this.configPath)));
            if (!(text.startsWith("# text added via config editor"))) {
                out.println("# text added via config editor");
            }
            out.println(text);
            out.close();
            Properties def = new Properties();
            try {
                StringReader sR = new StringReader(text);
                ByteArrayInputStream fiS = new ByteArrayInputStream(text
                        .getBytes());
                def.load(fiS);
            } catch (IOException ioE) {

            }
            // if we change the config, then update.
            setConfig(def);

        } catch (IOException ioE) {

        }
    }

    /**
     * saves a copy of the config data for the future searchs
     * 
     * @param indexDirName
     *            the index directory
     */
    private void writeIndexConfig(String indexDirName) {

        XHTMLviaDOM xhtml = new XHTMLviaDOM();
        Node body = xhtml.getBody();
        Properties attribs = new Properties();
        Properties liAttribs = new Properties();
        // list items I want to have a bit of separation.
        liAttribs
                .setProperty(
                        "style",
                        "margin-bottom:3ex;border-bottom-color:#D2691E;border-bottom-style:solid;border-bottom-width:medium;background-color:#FFF8DC;");

        Node h1 = xhtml.addKnot(body, "h1", attribs,
                "The following classes were used.");

        attribs.setProperty("style", "font-size:large;margin-bottom:2ex;");
        Node ulp = xhtml.addKnot(body, "p", attribs,
                "The class names, file suffixs and descriptions");
        attribs = new Properties();

        Enumeration keys = extraFileTypes.keys();
        Node ul = xhtml.addKnot(body, "ul", attribs, "");
        while (keys.hasMoreElements()) {
            String kagi = (String) keys.nextElement();

            try {
                Class c = Class.forName(extraFileTypes.getProperty(kagi));
                DocumentAnalyze uI = (DocumentAnalyze) c.newInstance();
                String info = uI.toString();
                int cls = info.indexOf(' ');
                String clsExtract = info.substring(0, cls);
                String descript = info.substring(cls + 1);

                Node li = xhtml.addKnot(ul, "li", liAttribs, "");
                attribs.setProperty("style", "margin:0ex;");
                Node p = xhtml.addKnot(li, "p", attribs, kagi);
                p = xhtml.addKnot(li, "p", attribs, clsExtract);
                attribs
                        .setProperty("style",
                                "margin-bottom:1ex;margin-top:0ex;margin-left:0ex;margin-right:0ex;");
                p = xhtml.addKnot(li, "p", attribs, descript);
            } catch (ClassNotFoundException cnfE) {
                System.out.println(cnfE.getMessage());
            } catch (InstantiationException iE) {
                System.out.println(iE.getMessage());
            } catch (IllegalAccessException iaE) {
                System.out.println(iaE.getMessage());
            }

        }

        xhtml.writeDOMtoFile(indexDirName + "/classes.html");

    }
}
