/*
 * Created on Aug 8, 2005
 *
 * Copyright 2005 Tom Louton; Berlin, Germany, http://www.theloutons.com/photoalbumtom.htm 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.theloutons.search.userinterfaces;

import com.theloutons.search.utils.*;
import java.awt.Button;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.awt.image.ImageFilter;
import java.awt.image.FilteredImageSource;
import java.awt.image.ReplicateScaleFilter;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringReader;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JRootPane;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.WindowConstants;
import javax.swing.border.Border;
import javax.imageio.ImageIO;

import java.util.Properties;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;

import org.w3c.dom.Node;

import com.theloutons.search.utils.ButtonwithToggle;
import com.theloutons.search.utils.StopWordMgmt;
import com.theloutons.search.utils.XHTMLviaDOM;

/**
 * @author Tom Louton
 * 
 *  
 */
public class UserSearchInterface extends JFrame {

    final Container contentPane = getContentPane();

    // the struts are used to put lines between the lables and boxes etc.
    private Component myStrut13 = Box.createVerticalStrut(13);

    private Component myStrut10 = Box.createVerticalStrut(10);

    private Component myStrut3 = Box.createVerticalStrut(3);

    // I have made these global so that I do not need to pass them between
    // methods, and to keep from creating lots of instances.
    final JButton indexButton = new JButton(
            "Find the directory in which the index is located");

    final JButton okButton = new JButton("Index now!");

    final JButton killButton = new JButton("Done");

    final JButton stopButton = new JButton("Exit");

    // final JButton parsedGo = new JButton("Parsed Query");
    final ButtonwithToggle parsedGo = new ButtonwithToggle("Parsed Query");

    // final JButton keyGo = new JButton("keyword Query");
    final ButtonwithToggle keyGo = new ButtonwithToggle("keyword Query");

    final ButtonwithToggle viewOutput = new ButtonwithToggle("View the output");

    final ButtonwithToggle indexInfo = new ButtonwithToggle("Index Information");

    final JButton outputButton = new JButton("Determine Output");

    final JButton getHelp = new JButton("Documentation");

    final JButton getConfig = new JButton("set configuration");

    final JFileChooser chooseIndexlocation = new JFileChooser();

    final JTextField indexDir = new JTextField(40);

    final JTextField searchText = new JTextField(40);

    final JTextField outFileName = new JTextField(40);

    final JLabel title = new JLabel("Parsed or Keyword Search");

    final JLabel dirLab = new JLabel("The directory where the index resides.");

    final JLabel searchLab = new JLabel("Please enter the search text");

    final JLabel outputLab = new JLabel(
            "Please check if this output file is ok.");

    final GridBagLayout gbl = new GridBagLayout();

    final GridBagConstraints gbc = new GridBagConstraints();

    File indFile = null;

    // String index;
    // location of Lucene logo
    String luceneLogo = "data/lucene.gif";

    int numDocs = 0;

    String configPath = System.getProperty("user.dir")
            + "/data/search.defaults";

    String textedit = "notepad";

    String mozilla = "C:/Program Files/Internet Explorer/iexplore.exe \"";

    JRootPane jrp = getRootPane();

    /**
     * This is the one to use.
     * 
     * @param title
     *            displayed in the windodw
     * @param path
     *            path to the /data/search.defaults
     * @param indexPath
     *            path to the directory where the index is.
     * @throws HeadlessException
     */
    public UserSearchInterface(String title, String path, String indexPath)
            throws HeadlessException {
        super(title);

        Properties def = new Properties();
        try {
            FileInputStream fiS = new FileInputStream(new File(path));
            def.load(fiS);
        } catch (IOException ioE) {
            System.out
                    .println("Either no config file or a problem has occurred.");
        }

        setConfig(def);
        setStartWindow();
        if (indexPath != null) {
            if ((new File(indexPath)).isDirectory()) {
                if (IndexReader.indexExists(indexPath)) {
                    indFile = new File(indexPath);
                    indexDir.setText(indexPath);
                    indexDir.setForeground(Color.BLUE);
                    keyGo.indexKnown();
                    parsedGo.indexKnown();
                    indexInfo.indexKnown();
                } else {
                    JOptionPane
                            .showMessageDialog(
                                    null,
                                    "The directory \""
                                            + indexPath
                                            + "\" does not contain an index! \n Change or use the Indexer tool");

                }

            }
        }
    }

    /**
     * This sets the actions for the various buttons Also sets up the start
     * window. And does the layout
     *  
     */
    private void setStartWindow() {
        jrp.setPreferredSize(new Dimension(650, 500));
        jrp.setBorder(BorderFactory
                .createTitledBorder("Move the Mouse to Move Duke"));
        jrp.setBorder(BorderFactory.createLineBorder(Color.black));
        Border paneEdge = BorderFactory.createEmptyBorder(0, 10, 10, 10);
        jrp.setBorder(paneEdge);
        /*
         * add a menu
         */
        addMenus();
        indexInfo.searchSet();
        /*
         * next the formatting for the text fields is set.
         */
        indexDir.setForeground(Color.RED);
        indexDir.setAlignmentX(Button.CENTER_ALIGNMENT);
        // sets the logo to my initials
        title.setIcon(setLogo("data/tkl.png", 40, 30));
        title.setFont(new Font("Times New Roman", Font.BOLD, 18));
        // let us add the Lucene logo to the right of title
        JLabel lucene = new JLabel(setLogo(luceneLogo, 150, 23));
        //"data/lucene.gif"
        setButtonActions();
        /*
         * Now we start the layout of the window.
         */
        contentPane.setLayout(gbl);
        gbc.anchor = GridBagConstraints.WEST;
        gbc.gridwidth = GridBagConstraints.RELATIVE;
        gbc.insets = new Insets(0, 0, 0, 0);
        contentPane.add(title, gbc);

        gbc.anchor = GridBagConstraints.EAST;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        contentPane.add(lucene, gbc);
        // block for input dir the dir containing the index.
        gbc.insets = new Insets(30, 0, 10, 0);
        contentPane.add(new JSeparator(), gbc);

        gbc.anchor = GridBagConstraints.WEST;
        gbc.insets = new Insets(0, 0, 0, 0);
        gbc.anchor = GridBagConstraints.LINE_START;
        contentPane.add(dirLab, gbc);
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.fill = GridBagConstraints.HORIZONTAL;
        contentPane.add(indexDir, gbc);

        gbc.insets = new Insets(5, 0, 0, 0);
        contentPane.add(new JSeparator(), gbc);

        gbc.insets = new Insets(0, 0, 0, 0);
        gbc.fill = GridBagConstraints.NONE;
        contentPane.add(indexButton, gbc);

        // block for setting the output file
        gbc.insets = new Insets(30, 0, 0, 0);
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        contentPane.add(new JSeparator(), gbc);

        gbc.insets = new Insets(5, 0, 0, 10);
        //	contentPane.add(new JSeparator(),gbc);

        gbc.insets = new Insets(0, 0, 0, 0);
        gbc.fill = GridBagConstraints.HORIZONTAL;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        contentPane.add(outputLab, gbc);

        gbc.insets = new Insets(0, 0, 0, 0);
        contentPane.add(outFileName, gbc);
        gbc.fill = GridBagConstraints.NONE;

        gbc.insets = new Insets(5, 0, 0, 0);
        contentPane.add(new JSeparator(), gbc);

        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.insets = new Insets(0, 0, 0, 0);
        contentPane.add(outputButton, gbc);

        gbc.insets = new Insets(30, 0, 0, 0);
        gbc.gridwidth = GridBagConstraints.RELATIVE;
        contentPane.add(new JSeparator(), gbc);

        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.insets = new Insets(20, 0, 0, 0);
        contentPane.add(searchLab, gbc);
        gbc.insets = new Insets(0, 0, 0, 0);

        gbc.fill = GridBagConstraints.HORIZONTAL;
        contentPane.add(searchText, gbc);
        gbc.fill = GridBagConstraints.NONE;

        gbc.insets = new Insets(5, 0, 0, 0);
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        contentPane.add(new JSeparator(), gbc);
        gbc.gridwidth = GridBagConstraints.RELATIVE;

        gbc.insets = new Insets(0, 0, 0, 0);
        // contentPane.add(keyGo, gbc);
        gbc.gridwidth = GridBagConstraints.RELATIVE;
        contentPane.add(parsedGo, gbc);
        gbc.insets = new Insets(0, 0, 0, 0);

        gbc.gridwidth = GridBagConstraints.REMAINDER;

        contentPane.add(viewOutput, gbc);
        gbc.insets = new Insets(20, 0, 0, 0);
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        contentPane.add(new JSeparator(), gbc);
        gbc.insets = new Insets(0, 0, 0, 0);

        gbc.gridwidth = GridBagConstraints.RELATIVE;
        gbc.anchor = GridBagConstraints.WEST;
        contentPane.add(indexInfo, gbc);

        gbc.gridwidth = GridBagConstraints.REMAINDER;
        contentPane.add(getHelp, gbc);
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        contentPane.add(getConfig, gbc);

        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.anchor = GridBagConstraints.EAST;
        stopButton.setBackground(Color.RED);
        contentPane.add(stopButton, gbc);

    }

    /**
     * here we add the menus
     */
    private void addMenus() {
        // put in a menu bar
        JMenuBar menuBar = new JMenuBar();
        JMenu menu = new JMenu("Menu");
        JMenuItem itemEnd = new JMenuItem("End Application NOW");
        itemEnd.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }
        });
        menu.add(itemEnd);
        menuBar.add(menu);
        JMenu helpMenu = new JMenu("Help");
        JMenuItem itemHelp = new JMenuItem("About private search");
        itemHelp.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
            
                JFrame info = new  FrameVersionInfo(mozilla);
                info.setLocation(30, 30);
                info.pack();
                info.setVisible(true);

                info.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);

                info.addWindowListener(new WindowAdapter() {
                    public void windowClosed(WindowEvent e) {
                       System.out.println("closed window");
                    }
                });

               // JOptionPane.showMessageDialog(null,
               //         "Private Search Version 1.0");
            }
        });
        helpMenu.add(itemHelp);
        menuBar.add(helpMenu);
        setJMenuBar(menuBar);

    }

    /**
     * This just sets the button actions for all the buttons makes it easier to
     * maintain and oversee
     */
    private void setButtonActions() {
        /**
         * now we add the action listner for the selection of the index. The
         * browse now so to speak.
         */
        indexButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {

                chooseIndexlocation.setMultiSelectionEnabled(false);
                chooseIndexlocation
                        .setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

                int state = chooseIndexlocation.showOpenDialog(null);
                indFile = chooseIndexlocation.getSelectedFile();
                try {
                    String indName = indFile.getCanonicalPath();
                    String indexPath = indexDir.getText();

                    if (indFile != null && state == JFileChooser.APPROVE_OPTION) {
                        // here we set up something
                        if (IndexReader.indexExists(indFile)) {
                            indexDir.setForeground(Color.BLUE);
                            indexDir.setText(indFile.getCanonicalPath());
                            keyGo.indexKnown();
                            parsedGo.indexKnown();
                            indexInfo.indexKnown();
                        } else {
                            //this is a rather poor example of prog.
                            // but I need to keep the file name for the user
                            // and sadly I use a file later
                            indexDir.setText(indexPath);
                            indFile = new File(indexPath);
                            JOptionPane
                                    .showMessageDialog(null,
                                            "This directory does not contain an index! \n Change or use the Indexer tool");
                        }
                    } else {
                        if (state == JFileChooser.CANCEL_OPTION) {
                            JOptionPane.showMessageDialog(null, "Canceled");
                        }
                        if (state == JFileChooser.ERROR_OPTION) {
                            JOptionPane.showMessageDialog(null, "Error!");
                        }
                    }
                } catch (IOException ioE) {
                    JOptionPane
                            .showMessageDialog(
                                    null,
                                    "Something is wrong with the directory you have choosen? \n Here is exception message"
                                            + ioE.getMessage());
                }
            }
        }); // action listner
        // parsedGo
        parsedGo.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                // set up the output file
                if (searchText.getText() == null) {
                    JOptionPane.showMessageDialog(null,
                            "Please enter search text.");
                    return;
                } else {
                    if (searchText.getText().length() < 1) {
                        JOptionPane.showMessageDialog(null,
                                "Please enter search text.");
                        return;
                    }
                    try {
                        IndexReader iR = IndexReader.open(indFile);
                        int numDocs = iR.numDocs();
                        iR.close();
                        Hits treffer = doParsedQuery("all", searchText
                                .getText());
                        putOutput(treffer, numDocs);
                        viewOutput.searchSet();
                        viewOutput.indexKnown();
                        JOptionPane
                                .showMessageDialog(null, "There were "
                                        + treffer.length()
                                        + " results to your search.");
                    } catch (IOException ioE) {
                        System.out.println("failed to get num of docs. "
                                + ioE.getMessage());
                    }
                }
            }

        });

        /**
         * To use the key word search keyGo button, need to get the text
         * searachText and then send it to the parser. Do not really need to
         * pass data as all vars. are central. * Not used anymore.
         */
        keyGo.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                // set up the output file
                if (searchText.getText() == null) {
                    JOptionPane.showMessageDialog(null,
                            "Please enter search text.");
                    return;
                } else {
                    if (searchText.getText().length() < 1) {
                        JOptionPane.showMessageDialog(null,
                                "Please enter search text.");
                        return;
                    }
                    try {
                        IndexReader iR = IndexReader.open(indFile);
                        int numDocs = iR.numDocs();
                        iR.close();

                        Hits treffer = doLayeredSearch("all", searchText
                                .getText());
                        putOutput(treffer, numDocs);
                        viewOutput.searchSet();
                        viewOutput.indexKnown();

                        JOptionPane
                                .showMessageDialog(null, "There were "
                                        + treffer.length()
                                        + " results to your search.");

                        // quickanddirtyOutput(treffer);
                    } catch (IOException ioE) {
                        System.out.println("failed to get num of docs. "
                                + ioE.getMessage());
                    }
                }
            }
        });
        /**
         * This opens the html file with data on the indexing how it was done
         * and file types.
         */
        indexInfo.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Runtime view = Runtime.getRuntime();
                try {
                    Process outHTM = view.exec(mozilla + indexDir.getText()
                            + "/classes.html" + "\"");
                } catch (IOException ioE) {
                    JOptionPane.showMessageDialog(null,
                            "There was a problem starting the exporere "
                                    + ioE.getMessage() + " Sorry.");
                    ioE.getStackTrace();
                }

            }
        });

        /**
         * now the exit button
         */
        stopButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                System.exit(0);
            }

        });
        /**
         * now the view output button
         */
        viewOutput.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Runtime view = Runtime.getRuntime();
                try {

                    Process outHTM = view.exec(mozilla
                            + outFileName.getText().trim() + "\"");
                } catch (IOException ioE) {
                    JOptionPane.showMessageDialog(null,
                            "There was a problem starting the exporere "
                                    + ioE.getMessage() + " Sorry.");
                    ioE.getStackTrace();
                }
            }

        });
        /**
         * now the view documents button
         */
        getHelp.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                Runtime view = Runtime.getRuntime();

                try {
                    Process outHTM = view.exec(mozilla
                            + System.getProperty("user.dir")
                            + "/data/indexerandsearcherusage.html#searching"
                            + "\"");
                } catch (IOException ioE) {
                    JOptionPane.showMessageDialog(null,
                            "There was a problem starting the exporere "
                                    + ioE.getMessage() + " Sorry.");
                    ioE.getStackTrace();
                }
            }

        });

        /**
         * now we add the action listner for the selection of the output file.
         * The browse now so to speak. Also as a default, set to the string
         * below.
         */
        outFileName.setText(System.getProperty("user.home")
                + System.getProperty("file.separator") + "searchresults.html");
        outputButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {

                chooseIndexlocation.setMultiSelectionEnabled(false);
                chooseIndexlocation
                        .setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

                int state = chooseIndexlocation.showOpenDialog(null);
                File output = chooseIndexlocation.getSelectedFile();

                if (output != null && state == JFileChooser.APPROVE_OPTION) {
                    // here we set up something
                    try {
                        String outName = output.getCanonicalPath();
                        outFileName.setForeground(Color.BLUE);
                        outFileName.setText(outName
                                + System.getProperty("file.separator")
                                + "searchresults.html");
                    } catch (IOException ioE) {
                        JOptionPane
                                .showMessageDialog(
                                        null,
                                        "Something is wrong with the directory you have chosen? \n Here is exception message"
                                                + ioE.getMessage());
                    }
                } else {
                    if (state == JFileChooser.CANCEL_OPTION) {
                        JOptionPane.showMessageDialog(null, "Canceled");
                    }
                    if (state == JFileChooser.ERROR_OPTION) {
                        JOptionPane.showMessageDialog(null, "Error!");
                    }
                }

            }
        }); // action listner
        /**
         * This is to ungrey the search start button when text for the search is
         * entered.
         */
        searchText.addKeyListener(new KeyListener() {
            public void keyPressed(KeyEvent ke) {
            }

            public void keyTyped(KeyEvent ke) {
                parsedGo.searchSet();
                keyGo.searchSet();
            }

            public void keyListener(KeyEvent ke) {

            }

            public void keyReleased(KeyEvent ke) {

            }
        });
        /**
         * Starts the configuration editor
         */
        getConfig.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                String initialData = getConfigData();
                final ConfigEditor configEdit = new ConfigEditor(
                        "The Configuration Editor", initialData,mozilla);
                configEdit.setLocation(30, 30);
                configEdit.pack();
                configEdit.setVisible(true);

                configEdit
                        .setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
                configEdit.addWindowListener(new WindowAdapter() {
                    public void windowClosed(WindowEvent e) {
                        boolean save = configEdit.saveOrNot();
                        if (save) {
                            // then write to file.
                            String text = configEdit.getText();
                            saveConfigData(text);
                        }
                    }

                });

            }
        });

    } // adding the action listeners.

    /**
     * This takes the search stream and constructes a "layered" seach
     * 
     * @param docType
     *            the type of document (pdf, txt, doc, html, or all) this is
     *            proforma here, but not activated.
     * @param searchWd
     *            the term to find
     * @return the results of the search as lucene Hits
     */
    private Hits doLayeredSearch(String docType, String searchWd) {

        StopWordMgmt def = new StopWordMgmt();

        String[] stopWords = def.getDefault();

        /**
         * use standard stop words
         */
        Analyzer analyzer = new StandardAnalyzer(stopWords);
        try {

            Directory directory = FSDirectory.getDirectory(indFile, false);

            IndexSearcher searcher = new IndexSearcher(directory);
            BooleanQuery layered = new BooleanQuery();

            TermQuery pdfType = new TermQuery(new Term("docType", "pdf"));
            if (!(docType.equals("all"))) {
                TermQuery searchdocType = new TermQuery(new Term("docType",
                        docType));
                layered.add(searchdocType, true, false);
            }

            TermQuery textQf = new TermQuery(new Term("contents", searchWd));
            layered.add(textQf, true, false);

            Hits treffer = searcher.search(layered);
            return searcher.search(layered);
        } catch (IOException ioE) {
            JOptionPane.showMessageDialog(null,
                    "There was an IO Exception for index directory in the key word query! "
                            + ioE.getMessage());
            return null;
        }
    } // end of doLayeredSearch

    /**
     * This is the search order.
     * 
     * @param docType
     *            I use "all" at the moment, it is though that such a search
     *            might be interesting if one says, for example, I would like
     *            only txt or pdf etc. To be realized later
     * @param parPhrase
     *            the search string
     * @return the Hits, the results of the search. lucene Hits
     */
    private Hits doParsedQuery(String docType, String parPhrase) {

        StopWordMgmt def = new StopWordMgmt();

        String[] stopWords = def.getDefault();
        Analyzer analyzer = new StandardAnalyzer(stopWords);
        Hits treffer = null;
        try {
            Directory directory = FSDirectory.getDirectory(indFile, false);

            IndexSearcher searcher = new IndexSearcher(directory);
            numDocs = searcher.maxDoc();

            QueryParser qPar = new QueryParser("contents", analyzer);

            qPar.setOperator(QueryParser.DEFAULT_OPERATOR_AND);
            Query parQu = QueryParser.parse(parPhrase, "contents", analyzer);

            treffer = searcher.search(parQu);
            return treffer;
        } catch (IOException ioE) {
            JOptionPane.showMessageDialog(null,
                    "There was an IO Exception for index directory in the parsed query! "
                            + ioE.getMessage());
            return null;
        } catch (ParseException pE) {
            JOptionPane.showMessageDialog(null, "There was a Parse Exception! "
                    + pE.getCause());
            return null;
        }

    }

    /**
     * This generates the html output. Note that I use a css forthe output.
     * Later, I will offer the user the chance to generate his own css.
     * 
     * @param treffer
     *            The reslts of either type of search.
     */
    private void putOutput(Hits treffer, int numDocs) {
        String css = getCSS();
        // prepare a string for CSS
        if (css == null) {
            css = "body {background-color:#7F7F7F; color:#000000;font-family:'Century Schoolbook',serif; font-size:12pt;letter-spacing:3px;padding:40px; border:double;}"
                    + "h1 {background-color:#FFBF00; color:#800000;font-family:'Century Schoolbook',serif; font-size:20pt;letter-spacing:3px;padding:10px; }"
                    + "h2 {background-color:#FFBF00; color:#800000;font-family:'Century Schoolbook',serif; font-size:18pt;letter-spacing:3px;padding:5px;}"
                    + "h3 {background-color:#00FF3F; color:#800000;font-family:'Century Schoolbook',serif; font-size:16pt;letter-spacing:3px;padding:5px;}"
                    + "h4{background-color:#FF7F00; color:#800000;font-family:'Century Schoolbook',serif; font-size:14pt;letter-spacing:3px;}"
                    + "h5 {background-color:#00FF3F; color:#800000;font-family:'Century Schoolbook',serif; font-size:12pt;letter-spacing:3px;}"
                    + "a {background-color:#FF9933; color:#5940BF;font-family:'Century Schoolbook',serif; font-size:12pt;letter-spacing:3px;}"
                    + "ul {background-color:#FF7F7F; color:#800000;font-family:'Century Schoolbook',serif; font-size:12pt;letter-spacing:3px;}"
                    + "ul li {background-color:#FFDF00; color:#800000;font-family:'Century Schoolbook',serif; font-size:12pt;letter-spacing:3px; border-color:#000000; border-style:solid; border-width:thick; margin-bottom:3em}"
                    + "ul li p {background-color:#C0C0C0; color:#800000;font-family:'Century Schoolbook',serif; font-size:12pt;letter-spacing:3px;}"
                    + "ul li ul {background-color:#FF7F00; color:#800000;font-family:'Century Schoolbook',serif; font-size:12pt;letter-spacing:3px;}"
                    + "ul li ul li {background-color:#C0C000; color:#800000;font-family:'Century Schoolbook',serif; font-size:12pt;letter-spacing:3px; border-color:#C000C0; margin-bottom:0em}"
                    + "ul li ul li p {background-color:#F0F0F0; color:#800F00;font-family:'Century Schoolbook',serif; font-size:12pt;letter-spacing:3px;}";
        }

        XHTMLviaDOM xhtml = new XHTMLviaDOM();
        putCSS(css);
        // put the reference here
        xhtml.addStyles("@import url(searchresults.css);");

        Node body = xhtml.getBody();
        Properties attribs = new Properties();

        // body = xhtml.addAttribs(body, attribs); this worked but now all with
        // css
        attribs = new Properties();
        // ######################################
        // ######################################

        if (treffer.length() > 0) {
            Node h1 = xhtml.addKnot(body, "h1", attribs,
                    "The results of your search for \"" + searchText.getText()
                            + "\"" + " follow. There were " + numDocs
                            + " documents indexed in this index.");
            Node h2 = xhtml.addKnot(body, "h2", attribs, "There were "
                    + treffer.length() + " results.");
            attribs = new Properties();
            Node ulbase = xhtml.addKnot(body, "ul", attribs, "");
            Properties tbcell = new Properties();
            Node h4;
            Node p;
            Node a;
            Node hr;
            Node liupp;
            Node ulunder;
            Node liunder;
            StringBuffer tmp;
            try {
                for (int lauf = 0; lauf < treffer.length(); lauf++) {

                    liupp = xhtml.addKnot(ulbase, "li", attribs,
                            "This is document No. " + lauf + "  ");
                    Properties fileAttrib = new Properties();

                    tmp = makeGoodHTML(new StringBuffer(treffer.doc(lauf).get(
                            "filename")));

                    fileAttrib.setProperty("href", "file://" + tmp);
                    p = xhtml.addKnot(liupp, "p", attribs, " ");
                    a = xhtml.addKnot(p, "a", fileAttrib, tmp.toString());
                    ulunder = xhtml.addKnot(liupp, "ul", attribs, "");
                    // here we have started the sub list for the document

                    if (treffer.doc(lauf).get("title") != null) {
                        tmp = makeGoodHTML(new StringBuffer(treffer.doc(lauf)
                                .get("title")));
                        liunder = xhtml.addKnot(ulunder, "li", attribs,
                                "The document had a Title:");
                        p = xhtml
                                .addKnot(liunder, "p", attribs, tmp.toString());
                    }
                    if (treffer.doc(lauf).get("summary") != null) {
                        tmp = makeGoodHTML(new StringBuffer(treffer.doc(lauf)
                                .get("summary")));
                        liunder = xhtml.addKnot(ulunder, "li", attribs,
                                "The document had a Summary:");
                        p = xhtml
                                .addKnot(liunder, "p", attribs, tmp.toString());
                    }
                    if (treffer.doc(lauf).get("synopsis") != null) {
                        tmp = makeGoodHTML(new StringBuffer(treffer.doc(lauf)
                                .get("synopsis")));
                        liunder = xhtml.addKnot(ulunder, "li", attribs,
                                "The document had a Synposis:");
                        p = xhtml
                                .addKnot(liunder, "p", attribs, tmp.toString());
                    }
                    // hr = xhtml.addKnot(liupp, "hr", null, "");

                }

                xhtml.writeDOMtoFile(outFileName.getText());
            } catch (IOException ioE) {

                System.out
                        .println("<h1 bgcolor=\"#FF0020\">Some sort of IO problem!!!"
                                + ioE.getMessage()
                                + outFileName.getText()
                                + "</h1>");

            }
        } else {
            Node h1 = xhtml.addKnot(body, "h1", attribs,
                    "The results of your search for " + searchText.getText()
                            + " follow.");
            Node h2 = xhtml.addKnot(body, "h2", attribs,
                    "There were 0 results.");

        }

    }

    /**
     * This just replaces less than and greater than with &gt; and &lt; These
     * symbols are often in the results.
     * 
     * @param strBuff
     *            the string to be output. Since I use XML I need to weed out a
     *            lot of chars. '\u0000' - '\u000C' and '\u0015' these are
     *            replace with -
     * @return the clean string buffer
     */
    private StringBuffer makeGoodHTML(StringBuffer strBuff) {
        for (int back = strBuff.length() - 1; back > -1; back--) {
            char test = strBuff.charAt(back);
            // this junk is just to replace > with &gt; etc.
            char[] forbidden = { '\u0000', '\u0001', '\u0002', '\u0003',
                    '\u0004', '\u0005', '\u0006', '\u0007', '\u0008', '\u0009',
                    '\u0015', '\u000B', '\u000C' };
            if (test == '<') {
                strBuff.insert(back, "&lt;");
                strBuff.deleteCharAt((back + 4));
            }
            if (test == '>') {
                strBuff.insert(back, "&gt;");
                strBuff.deleteCharAt((back + 4));
            }
            for (int lauf = 0; lauf < forbidden.length; lauf++) {
                if (test == forbidden[lauf]) {
                    strBuff.insert(back, "-");
                    strBuff.deleteCharAt((back + 1));
                }
            }
        }
        return strBuff;
    }

    /**
     * This is a quick and dirty output to check. Trouble shooting
     * 
     * @param treffer
     *            the search results.
     */
    private void quickanddirtyOutput(Hits treffer) {
        StringBuffer tmp;
        try {
            File outFile = new File(outFileName.getText());
            PrintWriter out = new PrintWriter(new FileWriter(outFile));
            for (int lauf = 0; lauf < treffer.length(); lauf++) {

                if (treffer.doc(lauf).get("title") != null) {
                    tmp = makeGoodHTML(new StringBuffer(treffer.doc(lauf).get(
                            "title")));
                    out.println(tmp);
                }
                if (treffer.doc(lauf).get("summary") != null) {
                    tmp = makeGoodHTML(new StringBuffer(treffer.doc(lauf).get(
                            "summary")));
                    out.println(tmp);
                }
                if (treffer.doc(lauf).get("synopsis") != null) {
                    tmp = makeGoodHTML(new StringBuffer(treffer.doc(lauf).get(
                            "synopsis")));
                    out.println(tmp);
                }
                if (treffer.doc(lauf).get("filename") != null) {
                    tmp = makeGoodHTML(new StringBuffer(treffer.doc(lauf).get(
                            "filename")));
                    out.println(tmp);
                }
            }
        } catch (IOException ioE) {

            System.out
                    .println("<h1 bgcolor=\"#FF0020\">Some sort of IO problem!!!"
                            + ioE.getMessage() + "</h1>");

        }
    }

    /**
     * 
     * @param imName
     *            file name for image
     * @param iw
     *            width of image (px)
     * @param ih
     *            hight of image (px)
     * @return the image as logo
     */
    private Icon setLogo(String imName, int iw, int ih) {
        Image im = null;
        try {
            Image logoIm = ImageIO.read(new File(imName));
            ImageFilter imFil = new ReplicateScaleFilter(iw, ih);
            FilteredImageSource imSour = new FilteredImageSource(logoIm
                    .getSource(), imFil);
            im = createImage(imSour);

        } catch (IOException ioE) {
            ioE.getStackTrace();
            return null;
        }
        return (Icon) new ImageIcon(im);

    }

    /**
     * A function to read the css file for the searchresults.
     * 
     * @return the contents of the file: data/searchresults.css
     */
    private String getCSS() {
        String css = null;
        try {
            BufferedReader in = new BufferedReader(new FileReader(
                    "data/searchresults.css"));
            String line;
            while ((line = in.readLine()) != null) {
                css = css + line;
            }
            if (css != null) {
                if (css.length() > 10) {
                    //  System.out.println(css);
                    return css;
                }
            }
            return null;
        } catch (FileNotFoundException fnfE) {
            return null;
        } catch (IOException ioE) {
            return null;
        }
    }

    /**
     * This writes the css to the directory where the search results are put.
     * 
     * @param css
     *            the css for the searchresults.
     */
    private void putCSS(String css) {

        try {
            int ind = outFileName.getText().lastIndexOf('.');
            String searchCss = outFileName.getText().substring(0, ind);
            PrintWriter out = new PrintWriter(new FileWriter(new File(searchCss
                    + ".css")));

            if (css != null) {
                out.println(css);
            }
            out.close();

            return;
        } catch (IOException ioE) {

        }
    }

    /**
     * Initiates the general configuration.
     * 
     * @param def
     *            properties with the configuration
     */
    public void setConfig(Properties def) {
        if (!(def.getProperty("mozilla.call") == null)) {
            mozilla = ((String) def.getProperty("mozilla.call")) + " \"";
        }
        if (!(def.getProperty("textedit.call") == null)) {
            textedit = ((String) def.getProperty("textedit.call")) + " \"";
        }
    }

    /**
     * This is used by the configuration editor.
     * 
     * @param text
     *            the contents of the configuration
     */
    private void saveConfigData(String text) {
        try {

            PrintWriter out = new PrintWriter(new FileWriter(new File(
                    this.configPath)));
            if (!(text.startsWith("# text added via config editor"))) {
                out.println("# text added via config editor");
            }
            out.println(text);
            out.close();
            Properties def = new Properties();
            try {
                StringReader sR = new StringReader(text);
                ByteArrayInputStream fiS = new ByteArrayInputStream(text
                        .getBytes());
                def.load(fiS);
            } catch (IOException ioE) {

            }
            // if we change the config, then update.
            setConfig(def);

        } catch (IOException ioE) {

        }
    }

    /**
     * 
     * @return the configuration
     */
    private String getConfigData() {
        String initialData = "";
        try {
            String line;
            BufferedReader bR = new BufferedReader(new FileReader(
                    this.configPath));
            while ((line = bR.readLine()) != null) {
                initialData = initialData + line + "\n";
            }
            return initialData;
        } catch (IOException ioE) {
            System.out.println("Problem with config data " + ioE.getMessage());
            return initialData;
        }

    }

}
