/**
 * Created on Jun 20, 2005
 * Copyright 2005 Tom Louton, Berlin, Germany, http://www.theloutons.com/photoalbumtom.htm 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * This is a reader and indexer for HTML, PDF, WinWordDoc and text file
 * html, doc, pdf and txt are the names in the index under docType
 * Edition 2 so to speak. We add the possibility to control this via 
 * a default config. (Java properties. If not present, defaults willbe
 * set. the means however, that one can control with out messing in the
 * program the types of files indexed.
 * 
 * 
 */
package com.theloutons.search.utils;


import java.io.File;
import java.io.PrintWriter;
import java.io.Reader;
import java.lang.Class;
import java.lang.ClassNotFoundException;
import java.util.Enumeration;
import java.util.Properties;

import org.apache.lucene.document.DateField;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;

import com.theloutons.search.specialreaders.DocumentAnalyze;

/**
 * @author Tom sg00756
 * 
 * This can be instantiated, but at the moment this is not meaningful.
 */
public class DocReadOrgainzer {

    private String docType;

    private long fileLn;

    private String name;

    private Reader theData;

    /**
     * in effect not needed. All static functions.
     */
    private DocReadOrgainzer(File f) {

    }

    /**
     * 
     * At the moment
     * 
     * @param f
     *            is the name of the file to be read. At the moment we look at
     *            the suffix to decide, html, doc (winword), pdf, txt, csv, rtf
     *            and ppt Before deciding what to do, add general properties to
     *            the doc. If we cannot handle the document, then the doc is set
     *            to document from this file to be returned to be indexed. If
     *            the file is not of a type which can be handled, null is
     *            returned.
     * @param f
     *            the file handle of the current document,,
     * @return the filled and indexed lucene document
     */
    public static Document getDocument(File f, Properties def, PrintWriter out) {
        org.apache.lucene.document.Document doc = null;
        if (f == null) {
            return null;
        }
        String name;
        name = f.getName();
        int index = name.lastIndexOf(".") + 1;
        String suffix;
        /**
         * in case the file name has no . in it, then adjust accordingly
         *  
         */
        if (index > 0) {
            suffix = name.substring(index);
        } else {
            suffix = "";
        }
        int fileLn = (int) f.length();

        /**
         * Here we have the suffix -- Then we will go through and read accoring
         * to type and get a standard return structure
         */

        /**
         * The docreader classes must extend DocumentAnalyze interface. to use
         * see below, instantiate it, call setFile and then get the doc from it.
         * I set afterward to null, just to be sure.
         */
        if (!(def == null)) {
            Enumeration keys = def.keys();
            while (keys.hasMoreElements()) {
                String kagi = (String) keys.nextElement();
                if (suffix.equalsIgnoreCase(kagi)) {
                    try {
                        Class c = Class.forName(def.getProperty(kagi));
                        DocumentAnalyze uI = (DocumentAnalyze) c.newInstance();
                        uI.setFile(f, out);
                        doc = uI.getDocument();
                        uI = null;
                        if (doc != null) {
                            doc.add(Field.Keyword("docType", kagi));
                        }
                    } catch (ClassNotFoundException cnfE) {
                        System.out.println(cnfE.getMessage());
                        doc = null;
                    } catch (InstantiationException iE) {
                        System.out.println(iE.getMessage());
                        doc = null;
                    } catch (IllegalAccessException iaE) {
                        System.out.println(iaE.getMessage());
                        doc = null;
                    }
                }
            }
        }

        /**
         * here we clean up the left overs so to speak.
         */
        if (!(doc == null)) {
            doc.add(Field.Keyword("filename", f.getAbsolutePath()));
            doc.add(Field.Keyword("lastModified", DateField.timeToString(f
                    .lastModified())));
            if (f.canWrite()) {
                doc.add(Field.Keyword("readOnly", "false"));
            } else {
                doc.add(Field.Keyword("readOnly", "true"));
            }
        }
        return doc;
    }

}
