/*
 * Created on Oct 25, 2005
 *
 * Copyright 2005 Tom Louton; Berlin, Germany, http://www.theloutons.com/photoalbumtom.htm 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.theloutons.search.utils;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
//import java.awt.GraphicsConfiguration;
import java.awt.HeadlessException;
import java.awt.Image;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.image.FilteredImageSource;
import java.awt.image.ImageFilter;
import java.awt.image.ReplicateScaleFilter;
import java.awt.event.ActionEvent;

import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JRootPane;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.HyperlinkEvent;
import javax.swing.text.html.HTMLFrameHyperlinkEvent;

/**
 * @author Tom Louton
 * 
 */

public class FrameVersionInfo extends JFrame implements HyperlinkListener,
        ActionListener {
    private static String version = "Portable Indexing, Searching System,\n"
            + " Version 2.1 \n" + "";

    private static String title = "Portable Index-Search System Info";

    //private String logoPath = "data/minchoukiirokyuudou.png";
    //private String logoPath = "data/aoikikyudou.png";
    // private String logoPath = "data/kuroukikzudou.png";
    //private String logoPath = "data/chakikyudou.png";
    //private String logoPath = "data/darkchairokyudou.png";
    private String logoPath = "data/kyuudouhgp.png";

    private String mozilla = "C:/Program Files/Internet Explorer/iexplore.exe \"";

    private Dimension logoDim = new Dimension(80, 50);

    private JEditorPane editPane;

    // data/swisschard.png

    /**
     * @param title
     * @throws java.awt.HeadlessException
     */
    public FrameVersionInfo(String mozilla) throws HeadlessException {
        super(title);
        showWindow();
       
    }

    /**
     * 
     * @param imName path the logo
     * @param iw width 
     * @param ih height
     * @param adopt get the graphic component for the logo
     * @return
     */
    private Icon setLogo(String imName, int iw, int ih, Component adopt) {
        Image im = null;
        try {
            Image logoIm = ImageIO.read(new File(imName));
            ImageFilter imFil = new ReplicateScaleFilter(iw, ih);
            FilteredImageSource imSour = new FilteredImageSource(logoIm
                    .getSource(), imFil);
            im = adopt.createImage(imSour);

        } catch (IOException ioE) {
            ioE.getStackTrace();
            return null;
        }
        return (Icon) new ImageIcon(im);

    }

    /**
     * Sets up the window for the text viewer
     *
     */
    private void showWindow() {

        GridBagLayout gbl = new GridBagLayout();

        GridBagConstraints gbc = new GridBagConstraints();
        Container contents = getContentPane();
        contents.setLayout(gbl);
        contents.setBackground(new Color(251, 237, 191));
        gbl.setConstraints(contents, gbc);
        JRootPane jrP = getRootPane();
        jrP.setPreferredSize(new Dimension(650, 500));
        Icon deco = setLogo(logoPath, logoDim.width, logoDim.height, contents);
        JLabel decoLb = new JLabel("KYUUDOU, seek enlightenment",deco,JLabel.CENTER);
        decoLb.setVerticalTextPosition(JLabel.BOTTOM);
        decoLb.setHorizontalTextPosition(JLabel.CENTER);
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        contents.add(decoLb, gbc);
        gbc.insets = new Insets(10, 0, 0, 0);
        JLabel ver = new JLabel(version);
        contents.add(ver, gbc);

        editPane = new JEditorPane();
        editPane.setEditable(false);
        editPane.addHyperlinkListener(this);
        editPane.setContentType("text/html");
        editPane.setText(this.getMessge());
        editPane.setSize(600, 500);
        
        contents.add(editPane, gbc);
    }

    /*
     * (non-Javadoc)
     * 
     * @see javax.swing.event.HyperlinkListener#hyperlinkUpdate(javax.swing.event.HyperlinkEvent)
     */
    public void hyperlinkUpdate(HyperlinkEvent e) {
        if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
            //JEditorPane pane = (JEditorPane) e.getSource();
            System.out.println(" class " + e.getClass().toString());
            if (e instanceof HTMLFrameHyperlinkEvent) {
                HTMLFrameHyperlinkEvent evt = (HTMLFrameHyperlinkEvent) e;
                System.out.println("HTML " + evt.getDescription());
                //  HTMLDocument doc = (HTMLDocument)pane.getDocument();
                //  doc.processHTMLFrameHyperlinkEvent(evt);
            }
            if (e instanceof HyperlinkEvent) {
                //System.out.println("HypelinkEvent " + e.getDescription());
                Runtime view = Runtime.getRuntime();

                try {
                    Process outHTM = view.exec(mozilla + e.getDescription()
                            + "\"");
                } catch (IOException ioE) {
                    JOptionPane.showMessageDialog(null,
                            "There was a problem starting the exporere "
                                    + ioE.getMessage() + " Sorry.");
                    ioE.getStackTrace();
                }

            } else {
                System.out.println(e.getDescription());
            }
        }
    }

    /*
     * (non-Javadoc) This does not seem to do anything.
     * 
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent event) {
        //  String url = startURL;
        Object source = event.getSource();
        System.out.println("inaction performed.");
    }

    /**
     * This is the main point of the class. This the 
     * message.
     * @return
     */
    private String getMessge() {
        String message = "<body bgcolor=\"#FBEDBF\" link=\"#FF0000\""
                + " alink=\"#FF0000\" vlink=\"#FF0000\">"
                + "<h1>The Portable Index-Search System</h1>"
                + "<p>Version 2.1</p>"
                + "<p>Copyright 2005 Tom Louton Licensed under the Apache License, Version 2.0 "
                + "<a href=\""
                + System.getProperty("user.dir")
                + "/data/tomlicense.html\">(the \"License\")</a></p>"
                + "<p>For more information<a href=\"http://www.theloutons.com/photoalbumtom.htm\"> see</a></p>"
                + "<div><h3>This system is based on the Apache <a href=\"http://lucene.apache.org/\">Lucene </a>project</h3></div>"
                + "<div>Various document readers were also used:"
                + "<ul type=\"none\">"
                + "<li>Apache Jakarta <a href=\"http://jakarta.apache.org/poi/index.html\">POI</a> for PowerPoint presentations and Excel tables</li> "
                + "<li>For HTML files <a href=\"http://tidy.sourceforge.net/\">JTidy</a></li> "
                + "<li>PDF files are parsed with <a href=\"http://www.pdfbox.org/\">PDF Box</a></li> "
                + "<li>WinWord documents use <a href=\"http://www.textmining.org/\">TextMining Org </a>tools</li> "
                + "</ul>" + "</div>" + "<p>System documentation is available "
                + "<a href=\"" + System.getProperty("user.dir")
                + "/data/indexerandsearcherusage.html" + "\">here.</a> </p>"
                + "<p> </p>" + "</body>";
        return message;
    }
}
