/*
 * Created on Sep 1, 2005
 *
 * Copyright 2005 Tom Louton, Berlin, Germany, http://www.theloutons.com/photoalbumtom.htm 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.theloutons.search.utils;

import java.io.Reader;
import java.io.StringReader;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;

/**
 * @author Tom Louton
 *
 */
public class SynopsisUtil {
    /**
     * 
     * @param text the extracted text from a document.
     * @return if found, the next 500 characters.
     */
    public static String getSynopsis(String text){
        String zusammen = null;
        int ind = 0;
        int lng = 0;
        int endSum = 0;
        if (text == null) {
            return zusammen;
        }
        int len = text.length();
        if (len < 50) {
            return text;
        }
        /**
         * Summary
         */
        ind = text.indexOf("Summary");
        if (ind > 0) {
            endSum = ind + 508;
            if (len < endSum) {
                endSum = lng - 1;
            }
            if ((ind + 8) > (endSum)) {
                return text;
            }
            return text.substring(ind + 8, endSum);
        }
        /**
         * if Purpose is present 
         */
        ind = text.indexOf("Purpose");
        if (ind > 0) {
            endSum = ind + 508;
            if (len < endSum) {
                endSum = lng - 1;
            }
            if ((ind + 8) > (endSum)) {
                return text;
            }
            return text.substring(ind + 8, endSum);
        }
        /**
         * check for Synopsis if there, get next 500 characters
         */
        ind = text.indexOf("Synopsis");
        if (ind > 0) {
            lng = text.substring(ind).length();
            endSum = ind + 508;
            if (len < endSum) {
                endSum = lng - 1;
            }
            if ((ind + 8) > (endSum)) {
                return text;
            }

            return text.substring(ind + 8, endSum);
        }
        /**
         * default, if none of the above present, 
         */
        /*
         * Set default to null now, as will get first 500 always
        ind = 0;
        lng = text.length();
        endSum = 500;
        if (len < endSum) {
            endSum = lng - 1;
        }
        return text.substring(ind, endSum);
        */
        return null;        
     }
    /**
     * Adds to the index 
     * @param docTxt the string containing the body of the document
     * @param doc the Lucene document
     * @return the document with index info added.
     */
    public static Document addString(String docTxt, Document doc) {
        if (docTxt == null) {
            return null;
        }
        Reader strReader = new StringReader(docTxt);
        doc.add(Field.Text("contents", strReader));
        /*
         * now get a summary, the first 500 words.
         */
        int lng = docTxt.length();
        int endSum = 500;
        if (lng < 500) {
            endSum = lng;
        }
        doc.add(Field.Keyword("summary", docTxt.substring(0, endSum)));
        //  doc.add(Field.Keyword("synopsis", getSynopsis(docTxt)));
        return doc;
    }

    

}
