/*
 * Created on Aug 9, 2005
 *
 * Copyright 2005 Tom Louton, Berlin, Germany, http://www.theloutons.com/photoalbumtom.htm 
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.theloutons.search.utils;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.*;
import javax.xml.parsers.*;
import org.apache.xml.serialize.*;
import org.xml.sax.SAXException;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.*;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Text;
/**
 * @author Tom Louton<p>
 * This class has the object of creating an empty XHTML DOM 
 * object and then has methods to add tags of various sorts.
 * There is a method to write it to a file. It can be instantiated
 * via file (a sort of template) or string (name of a file)
 * or null and then it starts from scratch.</p>
 * 
 */
public class XHTMLviaDOM {

 	private Comment comm;

	private Document dom;

	private Node body;

	private Node head;

	private Node style;

	private Node form = null;

	/**
	 * just creates an empty structured XHTM file
	 */
	public XHTMLviaDOM() {
		super();
		this.createDOM();
	}

	/**
	 * this reads in 
	 * @param f is the name of the file to be read in. If it does
	 * not exist or cannot be read an empty dom is created.
	 */
	public XHTMLviaDOM(File f) {
		super();
		if (f != null) {
			try {
				setDOM(new FileInputStream(f));
			} catch (FileNotFoundException fnfE) {
				this.createDOM();
				System.out.println("File not found ");
			}

		} else {
			this.createDOM();
			System.out.println("File was null ");
		}
	}
/**
 * 
 * @param fName the name of the file to be read in as template
 */
	public XHTMLviaDOM(String fName) {
		super();
		if (fName != null) {
			try {
				setDOM(new FileInputStream(new File(fName)));
			} catch (FileNotFoundException fnfE) {
				this.createDOM();
				System.out.println("File not found ");
			}
		} else {
			this.createDOM();
			System.out.println("String was null ");
		}
	}

	/**
	 * Helper to read the xml file 
	 * Origionally needed, but now not really
	 * @param iS the stream of xml data
	 * @return true if DOM object could be created from stream
	 */
	private boolean setDOM(InputStream iS) {
		try {
			DocumentBuilderFactory dFB = DocumentBuilderFactory.newInstance();
			DocumentBuilder dB = dFB.newDocumentBuilder();
			this.dom = dB.parse(iS);
			return true;
		} catch (ParserConfigurationException pE) {
			return false;
		} catch (IOException ioE) {
			System.out.println("IO Exception" + ioE.getMessage());
			return false;
		} catch (SAXException saxE) {
			System.out.println("SAX Exception" + saxE.getMessage());

			return false;
		}
	}

	/**
	 * creates a basic empty xhtml string
	 */
	private void createDOM() {
		DocumentBuilderFactory dBF = DocumentBuilderFactory.newInstance();
		try {
			DocumentBuilder dB = dBF.newDocumentBuilder();
			this.dom = dB.newDocument();
			DOMImplementation dImp = dom.getImplementation();
			DocumentType dType = dImp.createDocumentType("html",
					"-//W3C//DTD XHTML 1.0 Transitional//EN",
					"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd");
			dom = dImp.createDocument("http://www.com.theloutons", "html",
					dType);
			Element htm = this.dom.getDocumentElement();
			Attr att = this.dom.createAttribute("xmlns");
			att.setValue("http://www.w3.org/1999/xhtml");
			htm.setAttributeNode(att);
			Properties attribs = new Properties();
			/* <meta name="DC.Title" content="SELFHTML/Meta-Angaben"> <meta
			 * name="DC.Creator" content="Stefan M&uuml;nz"> <meta
			 * name="DC.Subject" content="Meta-Angaben"> <meta
			 * name="DC.Description" content="Heute bekannte Meta-Angaben in
			 * HTML"> <meta name="DC.Publisher" content="TeamOne"> <meta
			 * name="DC.Contributor" content="Wolfgang Nefzger"> <meta
			 * name="DC.Date" content="2001-12-15"> <meta name="DC.Type"
			 * content="Text"> <meta name="DC.Format" content="text/html"> <meta
			 * name="DC.Identifier" content="http://selfhtml.teamone.de/"> <meta
			 * name="DC.Source" content="HTML-Referenz"> <meta
			 * name="DC.Language" content="de"> <meta name="DC.Relation"
			 * content="Unterkapitel"> <meta name="DC.Coverage"
			 * content="Munich"> <meta name="DC.Rights" content="Alle Rechte
			 * liegen beim Autor">
			 */
			this.head = this.addKnot(htm, "head", null, null);
			this.addKnot(head, "title", null, "test ");
			attribs.setProperty("type", "text/css");
			this.style = this.addKnot(head, "style", attribs, null);
			this.comm = this.dom.createComment(" ");
			this.style.appendChild(this.comm);
			
			attribs = new Properties();
			attribs.setProperty("name", "D.C. Type");
			attribs.setProperty("content", "Text");
			this.addKnot(head, "meta", attribs, null);
			attribs.setProperty("name", "D.C. Author");
			attribs.setProperty("content", "Tom Louton and XHTML via DOM");
			this.addKnot(head, "meta", attribs, null);
			attribs.setProperty("name", "D.C. Author");
			attribs.setProperty("content", "Tom Louton and XHTML via DOM");
			this.addKnot(head, "meta", attribs, null);
			this.body = this.addKnot(htm, "body", null, null);

		} catch (ParserConfigurationException pcE) {

		}
	}
	
	/**
	 * 	/*
	 * adds a node to the dom. 
	 * @param nd is the node into which this will be written
	 * @param nodeName the name of the new node String
	 * @param attribs Java Properties contaings the atributes.
	 * 			The attribute name is the name of the property and 
	 * 			the value is the value of the key.
	 * @param contents String, the value of the node.
	 * @return the new node which was added to nd added.
	 */
	public Node addKnot(Node nd, String nodeName, Properties attribs,
			String contents) {
		Element piece;
		if (nodeName != null) {
			if (nodeName.length() > 0) {
				piece = this.dom.createElement(nodeName);
			} else {
				return null;
			}
		} else {
			return null;
		}
		if (attribs != null) {
			Enumeration nms = attribs.propertyNames();
			while (nms.hasMoreElements()) {
				String key = (String) nms.nextElement();
				Attr att = this.dom.createAttribute(key);
				att.setValue(attribs.getProperty(key));
				piece.setAttributeNode(att);
			}
		}
		attribs = null;
		Text txtNd = this.dom.createTextNode(contents);
		piece.appendChild(txtNd);
		return nd.appendChild(piece);
	}
	
	public Node addAttribs(Node nd,Properties attribs){
		if (attribs != null) {
			Element piece=(Element)nd;
			Enumeration nms = attribs.propertyNames();
			while (nms.hasMoreElements()) {
				String key = (String) nms.nextElement();
				Attr att = this.dom.createAttribute(key);
				att.setValue(attribs.getProperty(key));
				piece.setAttributeNode(att);
			}
		}
		return nd;
	}
	/**
	 * This just adds a comment to the styles tage
	 * @param isEverything the contents of the comment
	 */
	public void addStylesComment(String isEverything) {
		this.comm.appendData(isEverything);
		
	}
	/**
	 * 	
	 * adds styles to the css 
	 * @param isEverything  A String the styles to append CSS
	 */
	public void addStyles(String isEverything) {
		Node text =this.dom.createTextNode(isEverything);
		style.appendChild(text);

	}

	/**
	 * it is the resposibility of the user to get 
	 * the properties correctly.
	 * @param meta is a properties contianing name and content of the meta data 
	 * @return true if it worked.
	 */
	public boolean addMeta(Properties meta) {
		this.addKnot(this.head, "meta", meta, null);
		return true;
	}
	
	/**
	 * 	/*
	 * sets up a form, needed to get info back to the servlet
	 * @param attribs a set of attribs for the form
	 * @param url the address to be returned by the do get/put HTTP protocol
	 */
	public void addForm(Properties attribs, String url) {
		if (this.form != null) {
			return;
		}
		if (attribs == null) {
			attribs = new Properties();
		}
		attribs.setProperty("action", url);
		attribs.setProperty("method", "GET");
		this.form = addKnot(this.body, "form", attribs, null);
	}

	/**
	 * Adds a general text entry field to 
	 * the node nd (or form if nd is null)
	 * @param nd  the node in which to put the input
	 * @param name the name of the text node
	 * @param value the node value
	 * @param attribs list of attribs as a Java properties
	 * @return the new node which was added.
	 */
	public Node addInputText(Node nd, String name, String value,
			Properties attribs) {
		if (attribs == null) {
			attribs = new Properties();
		}
		attribs.setProperty("type", "text");
		attribs.setProperty("name", name);
		attribs.setProperty("value", value);
		if (nd != null) {
			return addKnot(nd, "input", attribs, null);
		}
		if (this.form != null) {
			return addKnot(form, "input", attribs, null);
		}
		return null;
	}

	/**
	 * this adds an input to the nd (if nd null, to the form) tag and gives it
	 * some standard attributes needed. @param nd the node to which to add the
	 * password input @param the name of the input @param value the value @param
	 * attibs any extra attibutes needed
	 * @param nd the node to which it is written
	 * @param name the name of the new node
	 * @param attribs attributes as Java Properties
	 * @return the new node which was added
	 */
	public Node addInputPassWd(Node nd, String name, Properties attribs) {
		if (attribs == null) {
			attribs = new Properties();
		}
		attribs.setProperty("type", "password");
		attribs.setProperty("name", name);
		if (nd != null) {
			return addKnot(nd, "input", attribs, null);
		}
		if (this.form != null) {
			return addKnot(form, "input", attribs, null);
		}
		return null;
	}

	/**
	 * This adds an input to the nd (if nd null, to the form) 
	 * tag and gives it some standard attributes needed. 
	 * @param nd the node to which to add the radio button
	 * @param name the name of the button
	 * @param value the value value field
	 * @param attribs Java attribs containing attributes for the 
	 * button
	 * @param cked true if the button is to be checked. If on has
	 * more than one button marked as checked, then the first is.
	 * @return the new node.
	 */
	public Node addInputRadio(Node nd, String name, String value,
			Properties attribs, boolean cked) {
		if (attribs == null) {
			attribs = new Properties();
		}
		attribs.setProperty("type", "radio");
		attribs.setProperty("name", name);
		attribs.setProperty("value", value);
		if (cked) {
			attribs.setProperty("checked", "");
		}
		if (nd != null) {
			return addKnot(nd, "input", attribs, null);
		}
		if (this.form != null) {
			return addKnot(form, "input", attribs, null);
		}
		return null;
	}

	/**
	 * This adds an check box to the nd (if nd null, to the form) tag
	 * and gives it some standard attributes needed.
	 * @param nd the node to which to add the check box
	 * @param name the name of the check box
	 * @param value the value for the value attribute
	 * @param attribs extra attribues (as Java Properties)
	 * @param cked true if to be checked.
	 * @return the new node, the check box
	 */
	public Node addInputCkBox(Node nd, String name, String value,
			Properties attribs, boolean cked) {
		if (attribs == null) {
			attribs = new Properties();
		}
		attribs.setProperty("type", "checkbox");
		attribs.setProperty("name", name);
		attribs.setProperty("value", value);
		if (cked) {
			attribs.setProperty("checked", "");
		}
		if (nd != null) {
			return addKnot(nd, "input", attribs, null);
		}
		if (this.form != null) {
			return addKnot(form, "input", attribs, null);
		}
		return null;

	}

	/**
	 * This adds a reset button to the nd (if nd null, to the form) 
	 * tag and gives it some standard attributes needed. 
	 * @param nd the node to which to add the reset button or the if null the form
	 * @param name the name of the reset button
	 * @param value the string value for the value attribute
	 * @param attribs extra attributes for this button
	 * @return the new node
	 */
	public Node addInputReset(Node nd, String name, String value,
			Properties attribs) {
		if (attribs == null) {
			attribs = new Properties();
		}
		attribs.setProperty("type", "reset");
		attribs.setProperty("name", name);
		attribs.setProperty("value", value);
		if (nd != null) {
			return addKnot(nd, "input", attribs, null);
		}
		if (this.form != null) {
			return addKnot(form, "input", attribs, null);
		}
		return null;

	}

	/**
	 * This adds a submitt button
	 * 	
	 * @param nd the node to which to add the button (usuall in a form)
	 * @param name the String containing the name of the button
	 * @param value value for the value attribute.
	 * @param attribs extra attributes as Java Properties.
	 * @return the new node
	 */
	public Node addInputSubmit(Node nd, String name, String value,
			Properties attribs) {
		if (attribs == null) {
			attribs = new Properties();
		}
		attribs.setProperty("type", "submit");
		attribs.setProperty("name", name);
		attribs.setProperty("value", value);
		if (nd != null) {
			return addKnot(nd, "input", attribs, null);
		}
		if (this.form != null) {
			return addKnot(form, "input", attribs, null);
		}
		return null;

	}

	/**
	 * Adds an inputbutton to the document where one wishes.
	 * Usually in the form if not node is given, then just in 
	 * the form
	 * @param nd the node to which the button is to be added.
	 * @param name the name of the button (String)
	 * @param value the value of the value attribute 
	 * @param attribs extra attributes (Java Properties)
	 * @return the new node
	 */
	public Node addInputButton(Node nd, String name, String value,
			Properties attribs) {
		if (attribs == null) {
			attribs = new Properties();
		}
		attribs.setProperty("type", "button");
		attribs.setProperty("name", name);
		attribs.setProperty("value", value);
		if (this.form != null) {
			return addKnot(form, "input", attribs, null);
		}
		return null;

	}

	/**
	 *  gets the body node (body tag)
	 * @return  the body node (body tag)
	 */
	public Node getBody() {
		return this.body;
	}

	/**
	 * @return the form node (form tag, I assume only one in the document)
	 */
	public Node getForm() {
		return this.form;
	}

	/**
	 * The method writes the xhtml document to a file. Must get a stream 
	 * writer too.
	 * @param fName the name of the file to be written 
	 * @return successful or not
	 */
	public boolean writeDOMtoFile(String fName) {
		try {
			OutputStream oS = new FileOutputStream(new File(fName));

			XMLSerializer xmlSer = new XMLSerializer(oS, null);		
			xmlSer.serialize(this.dom);
			oS.close();

			return true;
		} catch (FileNotFoundException fnfE) {
			System.out.println(fnfE.getMessage());
			return false;
		} catch (IOException ioE) {
			System.out.println("ioE " + ioE.getMessage());
			return false;
		}
	}

}
